<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/acting/Action.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
   
   
   
class ActionPipeElement extends PipeElement implements PipeElementInterface
{
       
       
   protected $gPipe;
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      $this->gGenericType = PipeElement::ACTION;
      $this->gSrc = $inSrc;
      $this->gType = $inType;
      $this->gComponent = $inComponent;
      if ( strlen( $inType ) == 0 ) {
           throw new UserException( "Must have a type attribute present", PalooseException::SITEMAP_ACTION_PARSE_ERROR, $inDOM );
        }
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      
      $pipeElements = $xpath->query( "*[ not( local-name() = 'parameter') ]" );
      $this->gPipe = new Pipe();
      foreach ( $pipeElements as $pipeNode ) {
         $name = $pipeNode->localName;
         
         switch ( $name ) {
               case "aggregate" :
                  try {
                     $aggregator = PipeElement::createAggregator( $pipeNode );
                     $this->gPipe->addComponent( $aggregator );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               
               case "generate" :
                  try {
                     $generator = PipeElement::createGenerator( $pipeNode );
                     $this->gPipe->addComponent( $generator );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
                  
               case "transform" :
                  try {
                     $transformer = PipeElement::createTransformer( $pipeNode );
                     $this->gPipe->addComponent( $transformer );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               case "serialize" :
                  try {
                     $serializer = PipeElement::createSerializer( $pipeNode );
                     $this->gPipe->addComponent( $serializer );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               
               case "read" :
                  try {
                     $reader = PipeElement::createReader( $pipeNode );
                     $this->gPipe->addComponent( $reader );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
            case "mount" :
               try {
                  $mount = PipeElement::createMount( $pipeNode );
                  $this->gPipe->addComponent( $mount );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "select" :
               try {
                  $selector = PipeElement::createSelector( $pipeNode );
                  $this->gPipe->addComponent( $selector );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "call" :
               try {
                  $call = PipeElement::createCall( $pipeNode );
                  $this->gPipe->addComponent( $call );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
               
            case "act" :
               try {
                  $action = PipeElement::createAction( $pipeNode );
                  $this->gPipe->addComponent( $action );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
               
            case "redirect-to" :
               if ( $inComponent->getPackageName() == 'AuthAction' or $inComponent->getPackageName() == 'SendMailAction' ) {
                    throw new UserException( "Cannot have a redirect within '" . $inComponent->getPackageName() . "'",
                       PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, $inDOM );
                  break;
               } else {
                    try {
                     $redirect = PipeElement::createRedirect( $pipeNode );
                     
                     $this->gPipe->addComponent( $redirect );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               }
         }
      
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      return $inDOM;
   }
   
   
   
   
  
   public function toString()
   {
      if ( $this->gPipe != null or $this->gParameters != NULL ) {
         $mess = "     <act type='{$this->gType}'>\n";
          if ( $this->gParameters != NULL ) {
            $mess .= $this->gParameters->toString();
         }
          if ( $this->gPipe != NULL ) {
            $mess .= $this->gPipe->toString();
         }
         $mess .= "     </act>\n";
         return $mess;
      } else {
         $mess = "   <act type='{$this->gType}'/>\n";
      }
   }
}
?>
