<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/gallery/GalleryImage.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/gallery/GalleryFolder.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/RunException.php" );
class GalleryIndex {
    
   private $gGalleryName;
    
   private $gGalleryBreadcrumbName;
    
   private $gGalleryDescription;
    
   private $gGalleryFolderList;
    
   private $gGalleryImageList;
    
   private $gCacheDirectory;
    
   
   
   
   public function __construct( DOMDocument $inDOM, $inGalleryPathName, _GalleryTransformer $inComponent )
   {
      
      $this->gCacheDirectory = StringResolver::expandPseudoProtocols( $inComponent->gCacheDirectory );
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "p", Environment::$configuration[ 'palooseNamespace' ] );  
      $catalogueNodeList = $xpath->query( "/p:gallery/p:name" );
      $this->gGalleryName = $catalogueNodeList->item(0)->nodeValue;
      
      $catalogueNodeList = $xpath->query( "/p:gallery/p:breadcrumb" );
      $this->gGalleryBreadcrumbName = $catalogueNodeList->item(0)->nodeValue;
      
      $catalogueNodeList = $xpath->query( "/p:gallery/p:description/*" );
      $this->gGalleryDescription = NULL;
      if ( $catalogueNodeList->length > 0 ) {
         $doc = new DOMDocument();
         $root = $doc->createElement( "description" );
         $root = $doc->appendChild( $root );
         foreach ( $catalogueNodeList as $node ) {
            $node = $doc->importNode( $node, true );
            $node = $root->appendChild( $node );
         }
         $this->gGalleryDescription = $doc;
      }
      
      if ( !file_exists( $this->gCacheDirectory ) ) {
          if ( mkdir( $this->gCacheDirectory, 0775 ) === false ) {
            throw new UserException(
               "Cannot create image cache: " . $this->gCacheDirectory, 
               PalooseException::GALLERY_ERROR, 
               NULL );
         } else {
            chmod( $this->gCacheDirectory, 0775 );
         }
      }
      $imageNodeList = $xpath->query( "/p:gallery/p:images/*" );
      if ( $imageNodeList->length > 0 ) {
         foreach ( $imageNodeList as $node ) {
            try {
               $image = new GalleryImage( $node, $inGalleryPathName, $inComponent );
               $this->gGalleryImageList[] = $image; 
            } catch( UserException $e ) {
               throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
            }
         }
      }
      $folderNodeList = $xpath->query( "/p:gallery/p:folders/*" );
      if ( $folderNodeList->length > 0 ) {
         foreach ( $folderNodeList as $node ) {
            $folder = new GalleryFolder( $node, $inGalleryPathName );
            $this->gGalleryFolderList[] = $folder;
         }
      }
   }
   
   
   public function getGalleryName()
   {
      return $this->gGalleryName;
   }
   
   
   public function getBreadcrumbName()
   {
      return $this->gGalleryBreadcrumbName;
   }
   
   
   
   public function getGalleryDescription()
   {
      return $this->gGalleryDescription;
   }
   
   
   public function getGalleryImageList()
   {
      return $this->gGalleryImageList;
   }
   
   
   public function getGalleryFolderList()
   {
      return $this->gGalleryFolderList;
   }
}
?>
