<?php
require_once( PALOOSE_LIB_DIRECTORY . "/gedcom/GedComReader.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/gedcom/GedComLine.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/gedcom/GedComNode.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/GedComException.php" );
    
class GedCom 
{
    
   
   private $gGedComFileName;
   
   private $gRoot;
   
   
   
    
   public function __construct( $inGedComFleName )
   {
       $this->gGedComFileName = $inGedComFleName;
   }
   
   
   
   public function processFile()
   {
      if ( file_exists( $this->gGedComFileName ) ) {
         
         try {
            $gedcomFile = new GedcomReader( $this->gGedComFileName );
            $this->gRoot = new GedcomNode( NULL );
            
            $nextLine = $gedcomFile->getNextLine();
            $nextNode = new GedcomNode( $nextLine );
            $finished = false;
            $nextNode->setLineNumber( $nextLine->getLineNumber() );
            
            while ( $nextNode->getLevel() != -1 && !$finished ) {
               $nextNode->setParentNode( $this->gRoot );
               $this->gRoot->addNode( $nextNode );
               if ( $nextNode->getTag() == "TRLR" ) {
                  $finished = true;
               } else {
                  
                  $nextNode = $nextNode->parse( $gedcomFile );
                  if ( $nextNode->getTag() == "TRLR" ) {
                     $finished = true;
                  }
               }
            }
            $domText = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n";
            return $domText . $this->gRoot->toXMLString();
         } catch ( GedComException $e ) {
              throw new GedComException( $e->getMessage(), $e->getCode() );
         }
      } else {
           throw new GedComException( "File '{$this->gGedComFileName}' does not exist", PalooseException::SITEMAP_GEDCOM_ERROR );
      }
   }
    
   
   
   
      
   
   
   
   
   
   
   
   public static function convertChar( $inChar )
   {
      if ( ord( $inChar ) >=0 and ord ( $inChar ) <= 128 ) return $inChar;
      return "&#" . ord( $inChar ) . ";";
   }
   
   
   
   public static function convertString( $inStr )
   {
      $str = htmlspecialchars( $inStr );
      $convertedStr = "";
      for ( $i = 0; $i <= strlen( $str ); $i++ ) {
         $convertedStr .= GedCom::convertChar( substr( $str, $i, 1 ) );
      }
      return $convertedStr;
   }
   
   
   
   public function toString()
   {
      return $this->gRoot->toString();
   }
   
   
   
   public function toXMLString()
   {
      return $this->gRoot->toXMLString();
   }
}
?>
