<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/caching/CacheFile.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/generation/GeneratorPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/gedcom/GedCom.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/GedComException.php" );
    
class GedComGenerator extends GeneratorPipeElement implements PipeElementInterface
{
    
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      parent::run( $inVariableStack, $inURL, $inQueryString, $inDOM );
      
      $fullFileName = StringResolver::expandString( $inVariableStack, $this->gSrc );
      $fullFileName = StringResolver::expandPseudoProtocols( $fullFileName );
      $cacheKey = CacheFile::getHash( $fullFileName );
      $isCachable = $this->gIsCachable;
      $useReqParams = Utilities::normalizeBoolean( $this->gComponent->getUseRequestParametersFlag() );
      
      
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $localCachable = $requestParameterModule->get( CacheFile::CACHABLE_ATTRIBUTE_NAME );
      if ( $useReqParams && $localCachable != NULL ) {
         $localCachable = Utilities::normalizeBoolean( $localCachable );
         $isCachable = $localCachable;
      }
      if ( $isCachable ) {
         $this->gCacheFile = new CacheFile( Environment::$configuration[ 'cacheDir' ] );
         $oldParametersHash = $this->gCacheFile->getParameterHash( $cacheKey );
         $newParameterHash = CacheFile::getHash( $this->gParameters->toString() );
         $validParameters = ( $oldParametersHash == $newParameterHash );
         
         
         $oldSrcTimeStamp = $this->gCacheFile->getTimeStamp( $cacheKey );
      } else {
         $oldParametersHash = 0;
         $validParameters = false;
         $oldSrcTimeStamp = 0;      
      }
      $documentDOM = new DomDocument();
      
      try {
         
         if ( file_exists( $fullFileName ) ) {
            $srcTimeStamp = filemtime( $fullFileName );
            $validSrcFile = ( $oldSrcTimeStamp == $srcTimeStamp );
            if ( $isCachable && $validSrcFile && $validParameters ) {
               
               $documentDOM = $this->gCacheFile->getCacheDOM( $cacheKey );
               $documentDOM->documentElement->setAttribute( "__file", $cacheKey );
               $documentDOM->documentElement->setAttribute( "__status", CacheFile::CACHE_VALID );
            } else {
               
               $gedcom = new GedCom( $fullFileName );
               $gedcom->processFile();
               $documentDOM->loadXML( rtrim( $gedcom->toXMLString() ) );
               
               if ( $isCachable ) {
                  $this->gCacheFile->updateDOM2Cache( $cacheKey, $srcTimeStamp, $documentDOM, $newParameterHash );
               }
               
               
               $documentDOM->documentElement->setAttribute( "__file", $cacheKey );
               
               $documentDOM->documentElement->setAttribute( "__status", CacheFile::CACHE_INVALID );
               
           }
         } else {
              throw new RunException( "File '$fullFileName' does not exist", PalooseException::SITEMAP_GENERATOR_RUN_ERROR );
         }
      } catch ( RunException $e ) {
        throw new RunException( $e->getMessage(), $e->getCode() );
      } catch ( GedComException $e ) {
        throw new RunException( $e->getMessage(), $e->getCode(), NULL );
     }
     return $documentDOM;
  }
   
   
   
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _GedComGenerator extends Generator {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "GedComGenerator";
   }
}
?>
