<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
class Mount extends PipeElement implements PipeElementInterface
{
    
   private $gURIPrefix;
    
   private $gSitemapSrc;
    
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      $this->gGenericType = PipeElement::MOUNT;
      $this->gSitemapSrc = $inSrc;
      $this->gType = $inType;
      $this->gComponent = $inComponent;
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $this->gURIPrefix = Utilities::getXPathListStringItem( 0, $xpath, "//m:mount/@uri-prefix" );
      
      if ( $this->gURIPrefix != NULL and 
               strlen( $this->gURIPrefix ) > 0  and
               substr( $this->gURIPrefix, strlen( $this->gURIPrefix ) - 1, 1 ) != Environment::getPathSeparator() ) {
         $this->gURIPrefix .= Environment::getPathSeparator();
      }
      if ( $this->gSitemapSrc == NULL or strlen( $this->gSitemapSrc ) == 0 ) {
         throw new UserException( "Must have a src attribute defined",
            PalooseException::SITEMAP_MOUNT_PARSE_ERROR,
            $inDOM );
       }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $subsitemap =  $this->gSitemapSrc;
      if ( ( strrpos( $subsitemap, Environment::getPathSeparator() ) + 1 ) == strlen( $subsitemap ) ) {
         
         $subsitemap .= "sitemap.xmap";
      }
      $subsitemap = StringResolver::expandString( $inVariableStack, $subsitemap );
      $subsitemap = StringResolver::expandPseudoProtocols( $subsitemap );
      
      
      $outputStream = new OutputStream( OutputStream::STANDARD_OUTPUT );
      try {
         
         
         $sitemapPathFileName = $subsitemap;
         $sitemapFileName = substr( $sitemapPathFileName, strrpos( $sitemapPathFileName, Environment::getPathSeparator() ) + 1 );
         $sitemapPathName = substr( $sitemapPathFileName, 0, strrpos( $sitemapPathFileName, Environment::getPathSeparator() ) );
      
         
         $sitemap = new Sitemap( $sitemapPathName, $sitemapFileName, Environment::$sitemapStack, $outputStream );
         $sitemap->parse();
         
         if ( ( strrpos( $this->gURIPrefix, Environment::getPathSeparator() ) + 1 ) == strlen( $this->gURIPrefix ) ) {
            
            $uriPrefix = $this->gURIPrefix;
         } else {
            $uriPrefix = $this->gURIPrefix . Environment::getPathSeparator();
         }
         $url = $inURL;
         
         if ( $this->gURIPrefix != NULL and strlen( $this->gURIPrefix ) > 0 ) {
            if ( strpos( $url, $this->gURIPrefix, 0 ) == 0 ) {
               $url = substr( $url, strlen( $this->gURIPrefix ) );
            }
         }
         
         
         $sitemap->run( $url, $inQueryString, false );
         
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode() );
      }
   }
   
   
   
   public function toString()
   {
      $mess = "    <mount src='{$this->gSitemapSrc}' package='" . get_class( $this ) . "'/>\n";
      return $mess;
   }
}
?>
