<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/gallery/GalleryIndex.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
 
class GalleryTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
   
   private $gRootDirectory;
   
   private $gRequestedImage;
   
   private $gGalleryDirectory;
   
   private $gNextImage;
   
   private $gPreviousImage;
    
   private $gTranslatedDOM;
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
        try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
      if ( strlen( $this->gParameters->getParameter( 'root' ) ) > 0 ) {
         $this->gRootDirectory = $this->gParameters->getParameter( 'root' );
      } else {
         $this->gRootDirectory = StringResolver::expandPseudoProtocols( $inComponent->gRootDirectory );
      }
      if ( strrpos( $this->gRootDirectory, Environment::getPathSeparator() ) < strlen( $this->gRootDirectory ) - 1 ) {
         
         $this->gRootDirectory = $this->gRootDirectory . Environment::getPathSeparator();
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $this->gDOM = $inDOM;
      
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $galleryDirectory = $requestParameterModule->get( 'src' );
      $this->gGalleryDirectory = $galleryDirectory;
      $this->gRequestedImage = $requestParameterModule->get( 'name' );
      $this->gNextImage = $requestParameterModule->get( 'next' );
      $this->gPrevImage = $requestParameterModule->get( 'prev' );
      $fullGalleryDirectory = $this->gRootDirectory . $galleryDirectory;
      if ( strlen( $galleryDirectory ) > 0 and strrpos( $fullGalleryDirectory, Environment::getPathSeparator() ) < strlen( $fullGalleryDirectory ) ) {
         $fullGalleryDirectory = $fullGalleryDirectory . Environment::getPathSeparator();
      }
      $rootNode = $inDOM->documentElement;
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      
      
      
      $this->processNode( $fullGalleryDirectory, $rootNode, $newElement );
      return $this->gTranslatedDOM;
   }
   
   
   
   private function processNode( $inGalleryDirectory, DOMNode $inParentNode, DOMNode $inNewParentNode )
   {
     
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = $attribute->name;
               $attribValue = $attribute->value;
            }
         }
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = $attribute->name;
               $attribValue = $attribute->value;
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         } else {
            
         }
      }
      
      
      $children = $inParentNode->childNodes;
      
      foreach ( $children as $child ) {
         if ( $child->nodeType == XML_ELEMENT_NODE ) {
         
            if ( $child->namespaceURI == Environment::$configuration[ 'palooseNamespace' ] and $child->localName == "gallery" ) {
               
               
               
               $rootDir = $child->getAttribute( "root" );
               if ( strlen( $rootDir ) > 0 ) {
                  $rootDir = StringResolver::expandPseudoProtocols( $rootDir );
                  if ( strlen( $rootDir ) > 0 and strrpos( $rootDir, Environment::getPathSeparator() ) < strlen( $rootDir ) ) {
                     $rootDir = $rootDir . Environment::getPathSeparator();
                  }
                  $fullGalleryDirectory = $rootDir . $this->gGalleryDirectory;
               } else {
                  $fullGalleryDirectory = $inGalleryDirectory;
               }
               
               
               
               $nodeName = $child->localName;
               $fullGalleryDirectory = StringResolver::expandPseudoProtocols( $fullGalleryDirectory );
               try {
                  if ( strlen( $this->gRequestedImage ) == 0 ) {
                     $this->processGallery( $fullGalleryDirectory, $child, $inNewParentNode );
                  } else {
                     $this->processImage( $fullGalleryDirectory, $child, $inNewParentNode );
                  }
               } catch ( UserException $e ) {
                  throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
               }
               
          } else {
              
              $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
              
              $this->processNode( $inGalleryDirectory, $child, $newElementNode );
              $inNewParentNode->appendChild( $newElementNode );
              }
         } else if ( $child->nodeType == XML_TEXT_NODE ) {
            
            $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
            $inNewParentNode->appendChild( $newTextNode );
         }
      }
   }
   
   
   
   
   private function processGallery( $inGalleryDirectory, DOMNode $inChildNode, DOMNode $inNewParentNode )
   {
      
      $galleryIndexName = $inGalleryDirectory . Environment::$configuration[ 'galleryIndex' ];
      if ( !file_exists( $galleryIndexName ) ) {
         throw new UserException( "Cannot find gallery index file: $galleryIndexName", 0, NULL );
      }
      $galleryIndexDOM = new DOMDocument();
      if ( !$galleryIndexDOM->load( $galleryIndexName ) ) {
         throw new UserException( $e->getMessage(), $e->getCode(), NULL );
      }
      try {
         $galleryIndex = new GalleryIndex( $galleryIndexDOM,
                                 $inGalleryDirectory,
                                 $this->gComponent );
          
      } catch ( UserException $e ) {
         throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
      }
      $palooseNS = $inChildNode->namespaceURI;
      $paloosePrefix = $inChildNode->prefix;
      
      $galleryRoot = $this->gTranslatedDOM->createElementNS( $palooseNS, $inChildNode->nodeName );
      $inNewParentNode->appendChild( $galleryRoot );
      
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":name" );
      $textNode = $this->gTranslatedDOM->createTextNode( $galleryIndex->getGalleryName() );
      $galleryElement->appendChild( $textNode );
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":dir" );
      $textNode = $this->gTranslatedDOM->createTextNode( $this->gGalleryDirectory );
      $galleryElement->appendChild( $textNode );
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":breadcrumb" );
      $breadcrumbArray = $this->generateBreadcrumbArray( $this->gRootDirectory, $this->gGalleryDirectory );
      if ( count( $breadcrumbArray ) > 0 ) {
         foreach ( $breadcrumbArray as $key => $crumb ) {
            $nameElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":name" );
            $attrNode = $this->gTranslatedDOM->createAttribute( "src" );
            $textNode = $this->gTranslatedDOM->createTextNode( $key );
            $attrNode->appendChild( $textNode );
            $nameElement->appendChild( $attrNode );
            
            $textNode = $this->gTranslatedDOM->createTextNode( $crumb );
            $nameElement->appendChild( $textNode );
            $galleryElement->appendChild( $nameElement );
         }
      }
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":description" );
      
      if ( $galleryIndex->getGalleryDescription() != NULL ) {
         $descriptionNodeList = $galleryIndex->getGalleryDescription()->documentElement->childNodes;
         if ( $descriptionNodeList != NULL and $descriptionNodeList->length > 0 ) {
            foreach( $descriptionNodeList as $node ) {
               $galleryElement->appendChild( $this->gTranslatedDOM->importNode( $node, 1 ) );
            }
         }
      }
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":folders" );
      $foldersList = $galleryIndex->getGalleryFolderList();
      if ( count( $foldersList ) > 0 ) {
         foreach ( $foldersList as $folder ) {
            $folderElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":folder" );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "src" );
            $textNode = $this->gTranslatedDOM->createTextNode( $folder->getRelativeFolderName() );
            $attrNode->appendChild( $textNode );
            $folderElement->appendChild( $attrNode );
   
            
            
            
            $descriptionNode = $folder->getDescription();
            if ( $descriptionNode != NULL ) {
                  $descNode = $this->gTranslatedDOM->importNode( $descriptionNode, 1 );
                  $folderElement->appendChild( $descNode );
                  $galleryElement->appendChild( $folderElement );
            } else {
               $textNode = $this->gTranslatedDOM->createTextNode( "" );
               $folderElement->appendChild( $textNode );
               $galleryElement->appendChild( $folderElement );
            }
         }
      }
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":images" );
      
      
      $attrNode = $this->gTranslatedDOM->createAttribute( "type" );
      $textNode = $this->gTranslatedDOM->createTextNode( "multi" );
      $attrNode->appendChild( $textNode );
      $galleryElement->appendChild( $attrNode );
   
      $imagesList = $galleryIndex->getGalleryImageList();
      if ( count( $imagesList ) > 0 ) {
         foreach ( $imagesList as $image ) {
            $imageElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":image" );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "img" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getCachedImageName() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "thumb" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getCachedThumbnailName() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "name" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getOriginalFileName() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "piclens" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getPicLensId() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            
            
            
            $descriptionNode = $image->getDescription();
            if ( $descriptionNode != NULL ) {
                  $descNode = $this->gTranslatedDOM->importNode( $descriptionNode, 1 );
                  $imageElement->appendChild( $descNode );
                  $galleryElement->appendChild( $imageElement );
            } else {
               $textNode = $this->gTranslatedDOM->createTextNode( "" );
               $imageElement->appendChild( $textNode );
               $galleryElement->appendChild( $imageElement );
            }
         }
      }
      $galleryRoot->appendChild( $galleryElement );
   }
   
   
   
   
   private function processImage( $inGalleryDirectory, DOMNode $inChildNode, DOMNode $inNewParentNode )
   {
      
      $galleryIndexName = $inGalleryDirectory . Environment::$configuration[ 'galleryIndex' ];
      if ( !file_exists( $galleryIndexName ) ) {
         throw new UserException( "Cannot find gallery index file: $galleryIndexName", 0, NULL );
      }
      $galleryIndexDOM = new DOMDocument();
      if ( !$galleryIndexDOM->load( $galleryIndexName ) ) {
         throw new UserException( $e->getMessage(), $e->getCode(), NULL );
      }
      try {
         $galleryIndex = new GalleryIndex( $galleryIndexDOM,
            $inGalleryDirectory,
            $this->gComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), NULL );
      }
 
      $palooseNS = $inChildNode->namespaceURI;
      $paloosePrefix = $inChildNode->prefix;
      
      $galleryRoot = $this->gTranslatedDOM->createElementNS( $palooseNS, $inChildNode->nodeName );
      $inNewParentNode->appendChild( $galleryRoot );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":name" );
      $textNode = $this->gTranslatedDOM->createTextNode( $galleryIndex->getGalleryName() );
      $galleryElement->appendChild( $textNode );
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":dir" );
      $textNode = $this->gTranslatedDOM->createTextNode( $this->gGalleryDirectory );
      $galleryElement->appendChild( $textNode );
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":breadcrumb" );
      $breadcrumbArray = $this->generateBreadcrumbArray( $this->gRootDirectory, $this->gGalleryDirectory );
      if ( count( $breadcrumbArray ) > 0 ) {
         foreach ( $breadcrumbArray as $key => $crumb ) {
            $nameElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":name" );
            $attrNode = $this->gTranslatedDOM->createAttribute( "src" );
            $textNode = $this->gTranslatedDOM->createTextNode( $key );
            $attrNode->appendChild( $textNode );
            $nameElement->appendChild( $attrNode );
            
            $textNode = $this->gTranslatedDOM->createTextNode( $crumb );
            $nameElement->appendChild( $textNode );
            $galleryElement->appendChild( $nameElement );
         }
      }
      $galleryRoot->appendChild( $galleryElement );
      $galleryElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":images" );
      
      $attrNode = $this->gTranslatedDOM->createAttribute( "type" );
      $textNode = $this->gTranslatedDOM->createTextNode( "single" );
      $attrNode->appendChild( $textNode );
      $galleryElement->appendChild( $attrNode );
      
      $attrNode = $this->gTranslatedDOM->createAttribute( "name" );
      $textNode = $this->gTranslatedDOM->createTextNode( $this->gRequestedImage );
      $attrNode->appendChild( $textNode );
      $galleryElement->appendChild( $attrNode );
   
      $imagesList = $galleryIndex->getGalleryImageList();
      if ( count( $imagesList ) > 0 ) {
         foreach ( $imagesList as $image ) {
            $imageElement = $this->gTranslatedDOM->createElementNS( $palooseNS, $paloosePrefix. ":image" );
            $attrNode = $this->gTranslatedDOM->createAttribute( "img" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getCachedImageName() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "thumb" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getCachedThumbnailName() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "name" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getOriginalFileName() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            $attrNode = $this->gTranslatedDOM->createAttribute( "piclens" );
            $textNode = $this->gTranslatedDOM->createTextNode( $image->getPicLensId() );
            $attrNode->appendChild( $textNode );
            $imageElement->appendChild( $attrNode );
   
            
            
            
            $descriptionNode = $image->getDescription();
            if ( $descriptionNode != NULL ) {
                 $descNode = $this->gTranslatedDOM->importNode( $descriptionNode, 1 );
                 $imageElement->appendChild( $descNode );
                 $galleryElement->appendChild( $imageElement );
            } else {
               $textNode = $this->gTranslatedDOM->createTextNode( "" );
               $imageElement->appendChild( $textNode );
               $galleryElement->appendChild( $imageElement );
            }
         }
      }
      $galleryRoot->appendChild( $galleryElement );
   }
   
   
   
   public function generateBreadcrumbArray( $inRoot, $inPath )
   {
      
      
      $levelPath = "";
      $breadCrumbPathArray[ "" ] = $inRoot;
      if ( strlen( $inPath ) > 0 ) {
         $pathArray = explode( Environment::getPathSeparator(), $inPath );
         foreach ( $pathArray as $path ) {
            $levelPath = Utilities::addTrailingSeparator( $levelPath ) . $path;
            $breadCrumbPathArray[ $levelPath ] = Utilities::addTrailingSeparator( $inRoot ) . $levelPath;
         }
      }
      
      
      
      
      
      
      $breadcrumbArray = NULL;
      
      $i = 0;
      foreach ( $breadCrumbPathArray as $key => $value ) {
         $i++;
         $value = Utilities::addTrailingSeparator( $value );
         $galleryIndexName = $value . Environment::$configuration[ 'galleryIndex' ];
         
         $galleryIndexDOM = new DOMDocument();
         $galleryIndexDOM->load( $galleryIndexName );
         
         
         $xpath = new domxpath( $galleryIndexDOM );
         $xpath->registerNamespace( "p", Environment::$configuration[ 'palooseNamespace' ] );  
         $breadcrumbArray[ $key ] = Utilities::getXPathListStringItem( 0, $xpath, "//p:breadcrumb" );
      }
      return $breadcrumbArray;
   }
   
   
   
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _GalleryTransformer extends Transformer {
    
   
    
   public $gRootDirectory;
   
    
   public $gCacheDirectory;
   
    
   public $gMaxThumbnailWidth;
   
    
   public $gMaxThumbnailHeight;
   
    
   public $gResizeImage;
   
    
   public $gMaxWidth;
   
    
   public $gMaxHeight;
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
       
      $this->gPackageName = "GalleryTransformer";
      
      $this->gRootDirectory = "";
      if ( strlen( $this->gParameters[ 'root' ] ) > 0 ) $this->gRootDirectory = $this->gParameters[ 'root' ];
      $this->gMaxThumbnailWidth = "";
      if ( strlen( $this->gParameters[ 'max-thumbnail-width' ] ) > 0 ) $this->gMaxThumbnailWidth = $this->gParameters[ 'max-thumbnail-width' ];
      $this->gMaxThumbnailHeight = "";
      if ( strlen( $this->gParameters[ 'max-thumbnail-height' ] ) > 0 ) $this->gMaxThumbnailHeight = $this->gParameters[ 'max-thumbnail-height' ];
      $this->gResizeImage = "";
      if ( strlen( $this->gParameters[ 'resize' ] ) > 0 ) $this->gResizeImage = $this->gParameters[ 'resize' ];
      $this->gMaxWidth = "";
      if ( strlen( $this->gParameters[ 'max-width' ] ) > 0 ) $this->gMaxWidth = $this->gParameters[ 'max-width' ];
      $this->gMaxHeight = "";
      if ( strlen( $this->gParameters[ 'max-height' ] ) > 0 ) $this->gMaxHeight = $this->gParameters[ 'max-height' ];
      $this->gCacheDirectory = "";
      if ( strlen( $this->gParameters[ 'image-cache' ] ) > 0 )
         $this->gCacheDirectory = StringResolver::expandPseudoProtocols( $this->gParameters[ 'image-cache' ] );
   }
 }
?>
