<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
class PageHitTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
   
   private $gCounterFileName;
   
   
   private $gIgnoreIP;
   
   const RETRIES = 10;
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
      $expandedString = StringResolver::expandPseudoProtocols( $inComponent->gCounterFileName );
      $this->gCounterFileName = $expandedString;
      $this->gIgnoreIP = $inComponent->gIgnoreIP;
      
      if ( strlen( $this->gParameters->getParameter( 'file' ) ) > 0 )
         $this->gCounterFileName = $this->gParameters->getParameter( 'file' );
      if ( strlen( $this->gParameters->getParameter( 'ignore' ) ) > 0 )
         $this->gIgnoreIP = $this->gParameters->getParameter( 'ignore' );
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $PageHitTransformFile = Environment::$configuration[ 'pageHitTransform' ];
      
      $expandedString = StringResolver::expandString( $inVariableStack, $PageHitTransformFile );
      $pageHitTransformFile = StringResolver::expandPseudoProtocols( $expandedString );
      
      $expandedString = StringResolver::expandString( $inVariableStack, $this->gCounterFileName );
      $this->gCounterFileName = StringResolver::expandPseudoProtocols( $expandedString );
      
      $xsl = new DomDocument(); 
      $xsl->load( $pageHitTransformFile ); 
      
      $proc = new XsltProcessor(); 
      $xsl = $proc->importStylesheet( $xsl );
      
      if ( $this->gComponent->getUseRequestParametersFlag() === true ) {
         $requestParameterModule = Environment::$modules[ 'request-param' ];
         $params = $requestParameterModule->getParams();
         foreach ( $params as $name => $value ) {
            $this->gParameters->setParameter( $name, $value );
         }
      }
      if ( sizeof( $this->gParameters->getParameterList() ) > 0  ) {
         $parameterList = $this->gParameters->getParameterList();
         foreach ( $parameterList as $name => $value ) {
            $expandedValue = StringResolver::expandString( $inVariableStack, $value );
            $expandedValue = StringResolver::expandPseudoProtocols( $expandedValue );
            if ( $proc->setParameter( null, $name, $expandedValue ) === false ) {
               throw new InternalException( "Problem setting parameter '$name' with value ' $value'", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
            }
         }
      }
      
      try {
         $hitCounter = $this->processHit( $inURL );
         if ( $proc->setParameter( null, "pageHits", $hitCounter ) === false ) {
            throw new InternalException( "Problem sending page hits parameter to transformer", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
         }
      } catch ( UserException $e ) {
          throw new UserException( "PageHit counter problem: \n" . $e->getMessage(), PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, $inDOM );
      } catch ( InternalException $e ) {
          throw new InternalException( $e->getMessage(), $e->getCode(), NULL );
      }
      return $proc->transformToDoc( $inDOM );
   }
   
   
   
   private function processHit( $inPageName )
   {
      
      $counterFile = NULL;
      if ( !file_exists( $this->gCounterFileName ) ) {
         $counterFile = fopen( $this->gCounterFileName, "x+" );
      } else {
         $counterFile = fopen( $this->gCounterFileName, "r+" );
      }
      if ( !isset( $counterFile ) ) {
         $this->gMess = "Could not open PageHit counter file: '" . $this->gCounterFileName . "'";
          throw new InternalError( "Could not open PageHit counter file: '" . $this->gCounterFileName . "'", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL ); 
      }
      
      $count = 0;
      $hitCounter = 0;
      $found = false;
      while ( $line = fgets( $counterFile, 4096 ) ) {
         $data = explode( ':', $line );
         if ( $data[ 0 ] == $inPageName ) {
            $found = true;
            $hitCounter = (int)$data[ 1 ] + 1;
            $count += strlen( $data[ 0 ] ) + 1;
            break;
         }
         $count += strlen( $line );
      }
      
      $ignoreIP = false;
      if ( isset( $this->gIgnoreIP ) ) {
         $ignoreIPList = explode( ',', $this->gIgnoreIP );
         foreach( $ignoreIPList as $ip ) {
            if ( $_SERVER[ 'REMOTE_ADDR' ] == trim( $ip ) ) {
               $ignoreIP = true;
               break;
            }
         }
      }
  
      
      if ( !$ignoreIP ) {
         $retriesCount = 0;
         $finished = false;
         while ( $retriesCount < self::RETRIES && !$finished ) {
            if ( flock( $counterFile, LOCK_EX ) ) { 
               
               if ( !$found ) {
                  $count = '0000000000000001';
                  fwrite( $counterFile, ( ( filesize( $this->gCounterFileName ) == 0 ) ? '' : "\r\n" ).$inPageName.':'.$count.':'.time());
                  $hitCounter = (int)$count;
               } else {
                  
                  fseek( $counterFile, $count );
                  fwrite( $counterFile, str_pad( $hitCounter, 16, '0', STR_PAD_LEFT ) );
               }
               flock( $counterFile, LOCK_UN );
               fclose( $counterFile );
               $finished = true;
            }
            $retriesCount ++;
         }
         if ( $retriesCount >= self::RETRIES && !$finished ) {
            throw new InternalException(
               "Cannot write page hit data (no lock after $retriesCount tries - permissions on directory/files?) : [{$this->gCounterFileName}]",
               PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR, NULL );
         }
      }
      return $hitCounter;
   }
   
   
   
   public function toString()
   {
      return parent::toString( $this->gGenericType );
   }
}
 
class _PageHitTransformer extends Transformer {
    
   
    
   public $gCounterFileName;
   
   
   public $gIgnoreIP;
  
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
       
      $this->gPackageName = "PageHitTransformer";
      $this->gCounterFileName = "";
      $this->gIgnoreIP = "";
      if ( array_key_exists( 'file', $this->gParameters ) ) {
         $this->gCounterFileName = $this->gParameters[ 'file' ];
      }
      if ( array_key_exists( 'ignore', $this->gParameters ) ) {
         $this->gIgnoreIP = $this->gParameters[ 'ignore' ];
      }
      
   }
}
?>
