<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/Transformer.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
 
class Actions {
   
   private $gActions;
      
   
   
   
    
   public function __construct()
   {
      $this->gActions = array();
   }
   
   
   
   public function parse( $inNode )
   {
      
      $actionsDom = new DOMDocument;
      $actionsDom->appendChild( $actionsDom->importNode( $inNode, 1 ) );
      
      $xpath = new domxpath( $actionsDom );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $component = $xpath->query( "//m:action" );
      
      
      foreach ( $component as $node ) { 
         $name = $node->getAttribute( "name" );
         $src = $node->getAttribute( "src" );
         $isCachable = Utilities::normalizeBoolean( $node->getAttribute( "cachable" ) );
         
         if ( !strlen( $name ) or !strlen( $src ) ) {
            $actionDom = new DOMDocument;
            $actionDom->appendChild( $actionDom->importNode( $node, 1 ) );
            throw new UserException( "Sitemap component action must have name and src attributes defined",
                                     PalooseException::SITEMAP_ACTION_PARSE_ERROR,
                                     $actionDom );
         }
         
         if ( strstr( $src, Environment::getPathSeparator() ) == 1 ) {
            $packageFilename = $src;
         } else if ( !strstr( $src, ":" . Environment::getPathSeparator() ) ) {
            
            $packageFilename = PALOOSE_LIB_DIRECTORY . Environment::getPathSeparator() . $src;
         } else {
            
            $packageFilename = StringResolver::expandPseudoProtocols( $src );
         }
         
         
         
         $package = Utilities::stripFileName( $packageFilename );
         
         $path = Utilities::stripPath( StringResolver::expandPseudoProtocols( $packageFilename ) );
         
         
         if ( array_key_exists( $name, $this->gActions ) ) {
            
            
         } else {
         }            
         
         require_once( $path . $package. ".php" );
         $actionName = "_" . $package;
         $action = new $actionName( $name, $path.$package, $node, $isCachable );
         
         try {
            $action->isValid();
            $this->gActions[ $name ] = $action;
         } catch( UserException $e ) {
            $actionDom = new DOMDocument;
            $actionDom->appendChild( $actionDom->importNode( $node, 1 ) );
            throw new UserException( $e->getMessage(), $e->getCode(), $actionDom );
         }
      }
   }
  
   
   
   public function getAction( $inName )
   {
      if ( !( array_key_exists( $inName, $this->gActions ) ) ) {
           throw new UserException( "Action name '". $inName . "' not found", PalooseException::SITEMAP_ACTION_PARSE_ERROR, NULL );
      } else {
         return $this->gActions[ $inName ];
      }
   }
  
   
   
   public function getActionClassName( $inType )
   {
      if ( !( array_key_exists( $inType, $this->gActions ) ) ) {
           throw new UserException( "Action type '". $inType . "' not found", PalooseException::SITEMAP_ACTION_PARSE_ERROR, NULL );
      } else {
         return $this->gActions[ $inType ]->getPackageName();
      }
   }
   
   
   
   public function toString()
   {
      $mess = "  <actions>\n";
      foreach ( $this->gActions as $node ) {
         $mess .= $node->toString();
      }
      $mess .= "  </actions>\n";
      return $mess;
   }
}
?>
