<?php
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
class CacheFile {
   
   private $gCacheDirectory;
   
   private $gCacheFileName;
   
   private $gCacheTime;
   
   private $gParameterHash;
   
   private $gHashData;
   
   private $gCacheEntryType;
   
   const CACHE_VALID = 1;
   const CACHE_INVALID = -1;
   
   
   const RETRIES = 10;
   
   const CACHE_FILE_NAME = 'paloose.cache';
   const CACHABLE_ATTRIBUTE_NAME = 'cachable';
   
   const CACHE_LINE_SEPARATOR = ",";
   const FILE_NAME_INDEX = 0;
   const FILE_MODIFY_TIME_INDEX = 1;
   const CACHED_FILE_NAME_INDEX = 2;
   const PARAMETER_HASH_INDEX = 3;
   
   
   
   public function __construct(  )
   {
  
      $rawCacheDirectory = Environment::$configuration[ 'cacheDir' ];
      $this->gCacheDirectory = StringResolver::expandPseudoProtocols( Environment::$configuration[ 'cacheDir' ] );
      $this->gCacheDirectory = Utilities::addTrailingSeparator( $this->gCacheDirectory );
      if ( !file_exists( $this->gCacheDirectory ) ) {
        if ( !mkdir( $this->gCacheDirectory ) ) {
          throw new InternalException(
            "Cannot create cache directory (permissions?) : {$rawCacheDirectory}[{$this->gCacheDirectory}]",
            PalooseException::SYSTEM_CACHE_ERROR, NULL );
        }
      }
      $this->gCacheFileName = $this->gCacheDirectory . self::CACHE_FILE_NAME;
      
      $this->gCacheTime = array();
      $this->gHashData = array();
      $this->gCacheEntryType = array();
      $this->gParameterHash = array();
      
      
      if ( file_exists( $this->gCacheFileName ) ) {
        $cacheHandle = fopen( $this->gCacheFileName, 'r' );  
         while ( !feof( $cacheHandle ) ) {
            $line = trim( fgets( $cacheHandle ) );
            if ( strlen( $line ) > 0 ) {
               $cacheData = explode( self::CACHE_LINE_SEPARATOR, $line );
               $cacheKey = $cacheData[ self::FILE_NAME_INDEX ];
               $this->gCacheTime[ $cacheKey ] = $cacheData[ self::FILE_MODIFY_TIME_INDEX ];
               $this->gHashData[ $cacheKey ] = $cacheData[ self::CACHED_FILE_NAME_INDEX ];
               $this->gParameterHash[ $cacheKey ] = $cacheData[ self::PARAMETER_HASH_INDEX ];
            }
         }
         fclose( $cacheHandle );
      }
   }
   
   
   
    
    public function getTimeStamp( $inKey )
    {
       if ( !array_key_exists( $inKey, $this->gCacheTime ) ) {
          return 0;  
       } else {
          return( $this->gCacheTime[ $inKey ] );
       }
    }
   
   
   
    
    public function getParameterHash( $inKey )
    {
       if ( !array_key_exists( $inKey, $this->gParameterHash ) ) {
          return 0;  
       } else {
          return( $this->gParameterHash[ $inKey ] );
       }
    }
   
   
   
    
   public function updateDOM2Cache( $inKey, $inTimeStamp, DOMDocument $inDom, $inParameterHash )
   {
      $this->gCacheTime[ $inKey ] = $inTimeStamp;
      $this->gParameterHash[ $inKey ] = $inParameterHash;
      $cachedFileName = $this->gCacheDirectory . $this->getHash( $inKey );
      
      $cacheHandle = fopen( $this->gCacheFileName, 'w' );  
      if ( $cacheHandle === false ) {
         throw new InternalException(
            "Cannot open cache file for writing: [{$this->gCacheFileName}]",
            PalooseException::SYSTEM_CACHE_ERROR, NULL );
       }
       
      $domHandle = fopen( $cachedFileName, 'w' );
      if ( $domHandle === false ) {
         throw new InternalException(
            "Cannot open cached file for writing: [{$cachedFileName}]",
            PalooseException::SYSTEM_CACHE_ERROR, NULL );
       }
       
       $count = 0;
       $finished = false;
       while ( $count < self::RETRIES && !$finished ) {
          if ( flock( $domHandle, LOCK_EX ) ) { 
             if ( flock( $cacheHandle, LOCK_EX ) ) { 
                foreach( $this->gCacheTime as $key => $timeStamp ) {
                   $line = $key . self::CACHE_LINE_SEPARATOR .
                             
                             $timeStamp . self::CACHE_LINE_SEPARATOR .
                             $this->getHash( $key ) . self::CACHE_LINE_SEPARATOR .
                             $this->gParameterHash[ $key ];
                   fwrite( $cacheHandle, $line . "\n" );
                }
                flock( $cacheHandle, LOCK_UN );
                fclose( $cacheHandle );
                fwrite( $domHandle, $inDom->saveXML() );
                flock( $domHandle, LOCK_UN );
                fclose( $domHandle );
                $finished = true;
             }
             $count++;
          }
       }
       if ( $count >= self::RETRIES && !$finished ) {
          throw new InternalException(
             "Cannot write cache data (no locks:$count) : [{$this->gCacheFileName}][{$this->getHash($key)}]",
             PalooseException::SYSTEM_CACHE_ERROR, NULL );
       }
    }
   
   
   
    
   public function getCacheDOM( $inKey )
   {
      $cachedFileName = $this->gCacheDirectory . $this->getHash( $inKey );
      $dom = new DOMDocument();
      if ( !$dom->load( $cachedFileName ) ) {
         throw new InternalException(
            "Cannot open cached file for reading: [$cachedFileName]",
            PalooseException::SYSTEM_CACHE_ERROR, NULL );
      }
      return $dom;
   }
   
   
   
    
    private function getDOMHash( DOMDocument $inDom )
    {
       return md5( $inDom->saveXML() );
    }
   
   
   
    
    public static function getHash( $inString )
    {
       return md5( $inString );
    }
   
   
   
    
    public function getCacheDir()
    {
       return $this->gCacheDirectory;
    }
   
   
   
    
    public function getCacheFileName()
    {
       return $this->gCacheFileName;
    }
}
?>
