<?php
class Utilities {
   
   
   public static function arrayToXML( $inArray, $inRootName )
   {
      $str = "<$inRootName>";
      foreach ( $inArray as $name => $value ) {
         $str .= "   <$name>$value</$name>";
      }
      $str .= "</$inRootName>";
      return $str;
   }
   
   
   public static function getXPathListNode( $inIndex, DOMXpath $inXPath, $inExpress )
   {
      $nodeList = $inXPath->query( $inExpress );
      if ( $nodeList == NULL or $nodeList->length == 0 ) return NULL;
      return $nodeList->item( $inIndex );
   }
   
   
   public static function getXPathListStringItem( $inIndex, DOMXpath $inXPath, $inExpression )
   {
      $nodeList = $inXPath->query( $inExpression );
      if ( $nodeList == NULL or $nodeList->length == 0 ) return "";
      return $nodeList->item( $inIndex )->textContent;
   }
   
   
   public static function stripPath( $inString )
   {
      $expStr = StringResolver::expandPseudoProtocols( $inString );
      $path = "";
      if ( strstr( $expStr, Environment::getPathSeparator() ) ) {
         $posOfLastSep = strrpos( $expStr, Environment::getPathSeparator() );
         $path = substr( $expStr, 0, $posOfLastSep + 1 );
      }
      return $path;
   }
   
   
   public static function stripFileName( $inString )
   {
      if ( strstr( $inString, Environment::getPathSeparator() ) ) {
         $posOfLastSep = strrpos( $inString, Environment::getPathSeparator() );
         return substr( $inString, $posOfLastSep + 1 );
      } else {
         return $inString;
      }
   }
   
   
   public static function stripExtension( $inString )
   {
      $posOfLastSep = strrpos( $inString, '.' );
      return substr( $inString, 0, $posOfLastSep ); 
   }
   
   
   public static function addTrailingSeparator( $inDirectory )
   {
      if ( $inDirectory != NULL and strlen( $inDirectory ) > 0 and strrpos( $inDirectory, Environment::getPathSeparator() ) < strlen( $inDirectory ) - 1 ) {
         return $inDirectory . Environment::getPathSeparator();
      } else {
         return $inDirectory;
      }
   }
   
   
   
     
    public static function join( $left, $right )
    {
        return rtrim( $left, '/\\' ) . DIRECTORY_SEPARATOR . ltrim( $right, '/\\') ;
    }
   
   
   public static function normalizeBoolean( $inBoolean )
   {
      if ( strnatcasecmp( $inBoolean, "true" ) == 0 or strnatcasecmp( $inBoolean, "yes" ) == 0 or $inBoolean === true  or $inBoolean == 1  ) {
         return 1;
      } else {
         return 0;
      }
   }
   
   
   
   public static function outputStringVerbatim( $inString )
   {
      echo "<pre>" . htmlspecialchars( $inString ) . "</pre>";
   }
   
   
   public static function makeStringHex( $inString, $inWithChar = false )
   {
      $str = "";
      for ( $i = 0; $i < strlen( $inString ); $i++ ) {
         $ch = substr( $inString, $i, 1 );
         if ( $inWithChar ) {
            $str .= "{" . $ch . ":" . bin2hex( $ch ) . "}";
         } else {
            $str .= "{" . bin2hex( $ch ) . "}";
         }
      }
      return $str;
   }
   
   
   
   
   public static function cdataEncode( $inData, $inTag )
   {
      if ( $inTag == "" ) {
         return $inData;
      }
      
      $inputString = $inData;
      $pattern = "/([\s\S]+)(<" . $inTag . "[^>]*>)([\s\S]+)(<\/" . $inTag . ">)([\s\S]+)/i";
      $result = preg_match( $pattern, $inputString, $matches, PREG_OFFSET_CAPTURE);
   
      if ( $result ) {
         $leftString = $matches[1][0];
         $startTag = $matches[2][0];
         $codeString = $matches[3][0];
         $endTag = $matches[4][0];
         $rightString = $matches[5][0];
         
         $patterns = array();
         $patterns[0] = '/&/';
         $patterns[1] = '/\</';
         $patterns[2] = '/\>/';
         $replacements = array();
         $replacements[2] = '&amp;';
         $replacements[1] = '&lt;';
         $replacements[0] = '&gt;';
         $codeString = preg_replace( $patterns, $replacements, $codeString );
         
         $replacementString = $leftString . $startTag. $codeString . $endTag . $rightString;
         $inputString = preg_replace( $pattern, $replacementString, $inputString ) . "\r";
      }
      return $inputString;
   }
   
   
   
   
   public function processCDATA( DOMDocument $inDOM, $inTag )
   {
      $newString = $inDOM->saveXML();
      
      if ( $inTag == "" ) {
         return $inDOM;
      }
      
      $newString = Utilities::cdataEncode( $newString, $inTag );
      $outputDocument = new DOMDocument();
      $outputDocument->loadXML( $documentString );
      return $outputDocument;
   }
   
   
   
   
    
   public static function correctURI()
   {
      
      $urlString = Environment::$gQueryString;
      $url = substr( $urlString, strpos( $urlString, "=" ) + 1 );
      Environment::$gRequestURI = $url;
      $newQueryString = "";
      if ( strpos( $url, "&" ) > 0 ) {
         $url = substr( $url, 0, strpos( $url, "&" ) );
         $newQueryString = substr( $urlString, strpos( $urlString, "&" ) + 1 );
      }
      Environment::$gQueryString = $newQueryString;
      return $url;
   }
   
   
   
    
   public static function xlateEntity( $inData )
   {
      $trans = array(
         '&Agrave;'=>'&#192;',    
         '&Aacute;'=>'&#193;',    
         '&Acirc;'=>'&#194;',     
         '&Atilde;'=>'&#195;',    
         '&Auml;'=>'&#196;',      
         '&Aring;'=>'&#197;',     
         '&AElig;'=>'&#198;',     
         '&Ccedil;'=>'&#199;',    
         '&Egrave;'=>'&#200;',    
         '&Eacute;'=>'&#201;',    
         '&Ecirc;'=>'&#202;',     
         '&Euml;'=>'&#203;',      
         '&Igrave;'=>'&#204;',    
         '&Iacute;'=>'&#205;',    
         '&Icirc;'=>'&#206;',     
         '&Iuml;'=>'&#207;',      
         '&ETH;'=>'&#208;',       
         '&Ntilde;'=>'&#209;',    
         '&Ograve;'=>'&#210;',    
         '&Oacute;'=>'&#211;',    
         '&Ocirc;'=>'&#212;',     
         '&Otilde;'=>'&#213;',    
         '&Ouml;'=>'&#214;',    
         '&Oslash;'=>'&#216;',    
         '&Ugrave;'=>'&#217;',    
         '&Uacute;'=>'&#218;',    
         '&Ucirc;'=>'&#219;',    
         '&Uuml;'=>'&#220;',    
         '&Yacute;'=>'&#221;',    
         '&THORN;'=>'&#222;',    
         '&szlig;'=>'&#223;',    
         '&agrave;'=>'&#224;',    
         '&aacute;'=>'&#225;',    
         '&acirc;'=>'&#226;',    
         '&atilde;'=>'&#227;',    
         '&auml;'=>'&#228;',    
         '&aring;'=>'&#229;',    
         '&aelig;'=>'&#230;',    
         '&ccedil;'=>'&#231;',    
         '&egrave;'=>'&#232;',    
         '&eacute;'=>'&#233;',    
         '&ecirc;'=>'&#234;',    
         '&euml;'=>'&#235;',    
         '&igrave;'=>'&#236;',    
         '&iacute;'=>'&#237;',    
         '&icirc;'=>'&#238;',    
         '&iuml;'=>'&#239;',    
         '&eth;'=>'&#240;',    
         '&ntilde;'=>'&#241;',    
         '&ograve;'=>'&#242;',    
         '&oacute;'=>'&#243;',    
         '&ocirc;'=>'&#244;',    
         '&otilde;'=>'&#245;',    
         '&ouml;'=>'&#246;',    
         '&oslash;'=>'&#248;',    
         '&ugrave;'=>'&#249;',    
         '&uacute;'=>'&#250;',    
         '&ucirc;'=>'&#251;',    
         '&uuml;'=>'&#252;',    
         '&yacute;'=>'&#253;',    
         '&thorn;'=>'&#254;',    
         '&yuml;'=>'&#255;',    
         '&emsp;'=>'&#x2003;',    
         '&ensp;'=>'&#x2002;',    
         '&emsp13;'=>'&#x2004;',    
         '&emsp14;'=>'&#x2005;',    
         '&numsp;'=>'&#x2007;',    
         '&puncsp;'=>'&#x2008;',    
         '&thinsp;'=>'&#x2009;',    
         '&hairsp;'=>'&#x200A;',    
         '&mdash;'=>'&#x2014;',    
         '&ndash;'=>'&#x2013;',    
         '&dash;'=>'&#x2010;',    
         '&blank;'=>'&#x2423;',    
         '&hellip;'=>'&#x2026;',    
         '&nldr;'=>'&#x2025;',    
         '&frac13;'=>'&#x2153;',    
         '&frac23;'=>'&#x2154;',    
         '&frac15;'=>'&#x2155;',    
         '&frac25;'=>'&#x2156;',    
         '&frac35;'=>'&#x2157;',    
         '&frac45;'=>'&#x2158;',    
         '&frac16;'=>'&#x2159;',    
         '&frac56;'=>'&#x215a;',    
         '&incare;'=>'&#x2105;',    
         '&block;'=>'&#x2588;',    
         '&uhblk;'=>'&#x2580;',    
         '&lhblk;'=>'&#x2584;',    
         '&blk14;'=>'&#x2591;',    
         '&blk12;'=>'&#x2592;',    
         '&blk34;'=>'&#x2593;',    
         '&marker;'=>'&#x25AE;',    
         '&cir;'=>'&#x25CB;',    
         '&squ;'=>'&#x25A1;',    
         '&rect;'=>'&#x25AD;',    
         '&utri;'=>'&#x25B5;',    
         '&dtri;'=>'&#x25BF;',    
         '&star;'=>'&#x2606;',    
         '&bull;'=>'&#x2022;',    
         '&squf;'=>'&#x25AA;',    
         '&utrif;'=>'&#x25B4;',    
         '&dtrif;'=>'&#x25BE;',    
         '&ltrif;'=>'&#x25C2;',    
         '&rtrif;'=>'&#x25B8;',    
         '&clubs;'=>'&#x2663;',    
         '&diams;'=>'&#x2662;',    
         '&hearts;'=>'&#x2661;',    
         '&spades;'=>'&#x2660;',    
         '&malt;'=>'&#x2720;',    
         '&dagger;'=>'&#x2020;',    
         '&Dagger;'=>'&#x2021;',    
         '&check;'=>'&#x2713;',    
         '&cross;'=>'&#x2717;',    
         '&sharp;'=>'&#x266F;',    
         '&flat;'=>'&#x266D;',    
         '&male;'=>'&#x2642;',    
         '&female;'=>'&#x2640;',    
         '&phone;'=>'&#x26E0;',    
         '&telrec;'=>'&#x2315;',    
         '&copysr;'=>'&#x2117;',    
         '&caret;'=>'&#x2041;',    
         '&lsquor;'=>'&#x201A;',    
         '&ldquor;'=>'&#x201E;',    
         '&fflig;'=>'&#xFB00;',    
         '&filig;'=>'&#xFB01;',    
         '&fjlig;'=>'fj',    
         '&ffilig;'=>'&#xFB03;',    
         '&ffllig;'=>'&#xFB04;',    
         '&fllig;'=>'&#xFB02;',    
         '&mldr;'=>'&#x2025;',    
         '&rdquor;'=>'&#x201D;',    
         '&rsquor;'=>'&#x2019;',    
         '&vellip;'=>'&#x22EE;',    
         '&hybull;'=>'&#x2043;',    
         '&loz;'=>'&#x2727;',    
         '&lozf;'=>'&#x2726;',    
         '&ltri;'=>'&#x25C3;',    
         '&rtri;'=>'&#x25B9;',    
         '&starf;'=>'&#x2605;',    
         '&natur;'=>'&#x266E;',    
         '&rx;'=>'&#x211E;',    
         '&sext;'=>'&#x2736;',    
         '&target;'=>'&#x2316;',    
         '&dlcrop;'=>'&#x230D;',    
         '&drcrop;'=>'&#x230C;',    
         '&ulcrop;'=>'&#x230F;',    
         '&urcrop;'=>'&#x230E;',    
         '&half;'=>'&#189;',    
         '&frac12;'=>'&#189;',    
         '&frac14;'=>'&#188;',    
         '&frac34;'=>'&#190;',    
         '&frac18;'=>'&#x215B;',    
         '&frac38;'=>'&#x215C;',    
         '&frac58;'=>'&#x215D;',    
         '&frac78;'=>'&#x215E;',    
         '&sup1;'=>'&#185;',    
         '&sup2;'=>'&#178;',    
         '&sup3;'=>'&#179;',    
         '&plus;'=>'+',    
         '&plusmn;'=>'&#xB1;',    
         '&equals;'=>'=',    
         '&divide;'=>'&#247;',    
         '&times;'=>'&#215;',    
         '&curren;'=>'&#164;',    
         '&pound;'=>'&#163;',    
         '&dollar;'=>'$',    
         '&cent;'=>'&#162;',    
         '&yen;'=>'&#165;',    
         '&num;'=>'#',    
         '&percnt;'=>'&#38;#37;',    
         '&ast;'=>'*',    
         '&commat;'=>'@',    
         '&lsqb;'=>'[',    
         '&bsol;'=>'\\',    
         '&rsqb;'=>']',    
         '&lcub;'=>'{',    
         '&horbar;'=>'&#x2015;',    
         '&verbar;'=>'|',    
         '&rcub;'=>'}',    
         '&micro;'=>'&#181;',    
         '&ohm;'=>'&#2126;',    
         '&deg;'=>'&#176;',    
         '&ordm;'=>'&#186;',    
         '&ordf;'=>'&#170;',    
         '&sect;'=>'&#167;',    
         '&para;'=>'&#182;',    
         '&middot;'=>'&#183;',    
         '&larr;'=>'&#x2190;',    
         '&rarr;'=>'&#x2192;',    
         '&uarr;'=>'&#x2191;',    
         '&darr;'=>'&#x2193;',    
         '&copy;'=>'&#169;',    
         '&reg;'=>'&#174;',    
         '&trade;'=>'&#8482;',    
         '&brvbar;'=>'&#xA6;',    
         '&not;'=>'&#xAC;',    
         '&sung;'=>'&#x266A;',    
         '&excl;'=>'!',    
         '&iexcl;'=>'&#xA1;',    
         '&apos;'=>'&#39;',    
         '&lpar;'=>'(',    
         '&rpar;'=>')',    
         '&comma;'=>',',    
         '&lowbar;'=>'_',    
         '&hyphen;'=>'&#x2010;',    
         '&period;'=>'.',    
         '&sol;'=>'/',    
         '&colon;'=>':',    
         '&semi;'=>';',    
         '&quest;'=>'?',    
         '&iquest;'=>'&#xBF;',    
         '&laquo;'=>'&#x2039;',    
         '&raquo;'=>'&#x203A;',    
         '&lsquo;'=>'&#x2018;',    
         '&rsquo;'=>'&#x2019;',    
         '&ldquo;'=>'&#x201C;',    
         '&rdquo;'=>'&#x201D;',    
         '&nbsp;'=>'&#160;',    
         '&shy;'=>'&#173;',    
         '&rcaron;'=>'&#x159;',  
         '&scaron;'=>'&#x353;',  
          
         '&euro;'=>'&#8364;'   
      );
      return strtr( $inData, $trans );
   }
   
   
   
    
   public static function is_utf8( $inString )
   {
      return ( preg_match( '/^([\x00-\x7f]|[\xc0-\xdf][\x80-\xbf]|[\xe0-\xef][\x80-\xbf]{2}|[\xf0-\xf7][\x80-\xbf]{3}|[\xf8-\xfb][\x80-\xbf]{4}|[\xfc-\xfd][\x80-\xbf]{5})*$/',
         $inString ) === 1);
   }
   
   
   
  
   
   public static function utf8ToUnicodeEntities( $inString )
   {
      
      
      $decrement[4] = 240;
      $decrement[3] = 224;
      $decrement[2] = 192;
      $decrement[1] = 0;
    
      
      $shift[1][0] = 0;
      $shift[2][0] = 6;
      $shift[2][1] = 0;
      $shift[3][0] = 12;
      $shift[3][1] = 6;
      $shift[3][2] = 0;
      $shift[4][0] = 18;
      $shift[4][1] = 12;
      $shift[4][2] = 6;
      $shift[4][3] = 0;
    
      $pos = 0;
      $len = strlen ( $inString );
      $encodedString = '';
      while ( $pos < $len ) {
         $singleLetter = false;
         $ch = substr( $inString, $pos, 1 );
         
         $asciiPos = ord( substr( $inString, $pos, 1 ) );
         
         if ( ( $asciiPos >= 240 ) && ( $asciiPos <= 255 ) ) {
            
            $thisLetter = substr($inString, $pos, 4 );
            $pos += 4;
         } else if ( ( $asciiPos >= 224 ) && ( $asciiPos <= 239 ) ) {
            
            $thisLetter = substr( $inString, $pos, 3 );
            $pos += 3;
         } else if (($asciiPos >= 192) && ($asciiPos <= 223)) {
            
            $thisLetter = substr( $inString, $pos, 2 );
            $pos += 2;
         } else {
            
            $thisLetter = substr( $inString, $pos, 1 );
            $pos += 1;
            $singleLetter = true;
         }
      
         
         if ( $singleLetter ) {
            $encodedString .= $ch;
         } else {
            $thisLen = strlen ( $thisLetter );
            $thisPos = 0;
            $decimalCode = 0;
            while ( $thisPos < $thisLen ) {
               $thisCharOrd = ord( substr( $thisLetter, $thisPos, 1 ) );
               if ( $thisPos == 0 ) {
                  $charNum = intval( $thisCharOrd - $decrement[$thisLen] );
                  $decimalCode += ( $charNum << $shift[$thisLen][$thisPos] );
               } else {
                  $charNum = intval ($thisCharOrd - 128);
                  $decimalCode += ( $charNum << $shift[$thisLen][$thisPos] );
               }
               $thisPos++;
            }
            if ( $thisLen == 1 )
               $encodedLetter = "&#". str_pad($decimalCode, 3, "0", STR_PAD_LEFT) . ';';
            else
               $encodedLetter = "&#". str_pad($decimalCode, 5, "0", STR_PAD_LEFT) . ';';
            $encodedString .= $encodedLetter;
         }
    } 
    return $encodedString;
   }
}
?>
