<?php
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/RunException.php" );
class GalleryImage {
    
   private $gDescriptionNode;
    
   private $gPicLensId;
    
   private $gCacheDirectory;
    
   private $gFileName;
    
   private $gFullFileName;
    
   private $gCachedThumbnailName;
    
   private $gCachedImageName;
    
   
   
   
    
   public function __construct( DOMNode $inNode, $inGalleryPathName, _GalleryTransformer $inComponent )
   {
      $dom = new DOMDocument;
      $dom->appendChild( $dom->importNode( $inNode, 1 ) );
      $this->gCacheDirectory = StringResolver::expandPseudoProtocols( $inComponent->gCacheDirectory );
      $xpath = new domxpath( $dom );
      $xpath->registerNamespace( "p", Environment::$configuration[ 'palooseNamespace' ] );  
     
      $this->gFullFileName = Utilities::getXPathListStringItem( 0, $xpath, "/p:image/@src" );
      $this->gFileName = Utilities::getXPathListStringItem( 0, $xpath, "/p:image/@src" );
      if ( strlen ( $this->gFullFileName ) == 0 ) {
         throw new UserException( 
            "Must have an associated src attribute with gallery image",
            PalooseException::SITEMAP_GALLERY_PARSE_ERROR,
            $dom );
      }
      
      $this->gDescriptionNode = Utilities::getXPathListNode( 0, $xpath, "/p:image/p:description" );
    
      $this->gPicLensId = Utilities::getXPathListStringItem( 0, $xpath, "/p:image/@piclens" );
      $this->gFullFileName = $inGalleryPathName . $this->gFullFileName;
      
      $parts = pathinfo( $this->gFullFileName );
      $imageType = $parts[ "extension" ];
      $this->gCachedImageName = $this->generateNewCacheImageName( $this->gFullFileName ); 
      $this->gCachedThumbnailName = $this->generateNewCacheThumbnailName( $this->gFullFileName ); 
      
       if ( !file_exists( $this->gFullFileName ) ) {
         $msg = "Could not find image file: '" . $this->gFullFileName . "'";
           throw new UserException( $msg, PalooseException::GALLERY_ERROR, NULL );
      }
      if ( !file_exists( $this->gCacheDirectory . $this->gCachedImageName ) ) {
          try {
             $this->generateNewImage( $this->gCacheDirectory . $this->gCachedImageName,
                (int)($inComponent->gMaxWidth), (int)($inComponent->gMaxHeight) );
         } catch( UserException $e ) {
            throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
         }
      }
      if ( !file_exists( $this->gCacheDirectory . $this->gCachedThumbnailName ) ) {
          try {
             $this->generateNewImage( $this->gCacheDirectory . $this->gCachedThumbnailName,
                (int)($inComponent->gMaxThumbnailWidth), (int)($inComponent->gMaxThumbnailHeight) );
          } catch( UserException $e ) {
            throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
         }
      }
   }
   
   
   
   
    
   public function generateNewImage( $inFileName, $inMaxWidth, $inMaxHeight  )
   {
 
      $parts = pathinfo( $this->gFullFileName );
      $imageType = $parts[ "extension" ];
      if ( function_exists( "imagecreatefromjpeg" ) ) {
         switch( $imageType ) {
              case "jpeg":
            case "jpg":
                  $originalImage = imagecreatefromjpeg( $this->gFullFileName );
                  break;
            case "gif":
                  $originalImage = imagecreatefromgif( $this->gFullFileName );
                  break;
            case "png":
                  $originalImage = imagecreatefrompng( $this->gFullFileName );
                  break;
         }
   
         $origWidth = imagesx( $originalImage );
         $origHeight = imagesy( $originalImage );
         
         $xRatio = $inMaxWidth/( $origWidth );
         $yRatio = $inMaxHeight/( $origHeight );
   
         
         touch( $inFileName );      
   
         
         
         if ( $xRatio > 1 && $yRatio > 1 ) {
            imagejpeg( $originalImage, $inFileName );
            chmod( $inFileName, 0775 );
            imagedestroy( $originalImage );
            return;
           }
   
           
           if ( $xRatio < $yRatio )
              $scaleRatio =  $xRatio;
           else
              $scaleRatio =  $yRatio;
           $newWidth = (int)( $origWidth * $scaleRatio );
           $newHeight = (int)( $origHeight * $scaleRatio );
         $newImage = imagecreatetruecolor( $newWidth, $newHeight );
         imagecopyresampled( $newImage, $originalImage, 0, 0, 0, 0, $newWidth, $newHeight, $origWidth, $origHeight );
         imagejpeg( $newImage, $inFileName );
         chmod( $inFileName, 0775 );
         imagedestroy( $newImage );
         imagedestroy( $originalImage );
      } else {
         
         $cmd = Environment::$configuration[ 'ImageMagickBin' ] . "identify " . $this->gFullFileName;
         $err = false;
         $output = "";
         exec( "$cmd 2>&1", $output, $err );
         if ( $err ) {
            throw new UserException(
               "Could not run image info for " . $this->gFullFileName . " : " . $output . "[$cmd]",
               PalooseException::GALLERY_ERROR,
               NULL );
          }
         $infoString = $output[0];
         
         
         
         
         $sizePattern = "/(\d\d*)x(\d\d*)/";
         if ( preg_match( $sizePattern, $infoString, $regs ) ) {
            $origWidth = $regs[1];
            $origHeight = $regs[2];
         } else {
            throw new UserException(
               "Invalid info format: " . $infoString,
               PalooseException::GALLERY_ERROR,
               NULL );
         }
         $xRatio = $inMaxWidth/( $origWidth );
         $yRatio = $inMaxHeight/( $origHeight );
   
         
         touch( $inFileName );      
        
         
         
         if ( $xRatio > 1 && $yRatio > 1 ) {
            if ( !copy( $this->gFullFileName, $inFileName ) ) {
               throw new UserException(
                  "Could not copy " . $this->gFullFileName . " to " . $inFileName, 
                  PalooseException::GALLERY_ERROR,
                  NULL );
            };
            chmod( $inFileName, 0775 );
            return;
           }
   
         
           if ( $xRatio < $yRatio )
              $scaleRatio =  $xRatio;
           else
              $scaleRatio =  $yRatio;
         $scaleRatio =  $scaleRatio * 100;  
         $cmd = Environment::$configuration[ 'ImageMagickBin' ] . "convert " . $this->gFullFileName . " -resize " . $scaleRatio . "% " . $inFileName;
         exec( "$cmd 2>&1", $output, $err );
         if ( $err ) {
            throw new UserException(
               "Could not run image info for " . $this->gFullFileName . " : " . $output,
               PalooseException::GALLERY_ERROR,
               NULL );
          }
      }
   }
   
   
   
    
   public function generateNewCacheImageName( $inFileName )
   {
      $hash = md5( $inFileName );
      $parts = pathinfo( $inFileName );
      $newName = $hash . "." . $parts[ "extension" ];
      return $newName;
   }
   
   
   
    
   public function generateNewCacheThumbnailName( $inFileName )
   {
      $hash = md5( $inFileName );
      $parts = pathinfo( $inFileName );
      $newName = $hash . "-th." . $parts[ "extension" ];
      return $newName;
   }
   
   
   public function getDescription()
   {
        return $this->gDescriptionNode;
   }
   
   
   public function getPicLensId()
   {
      return $this->gPicLensId;
   }
   
   public function getCacheDirectory()
   {
      return $this->gCacheDirectory;
   }
   
   public function getOriginalFileName()
   {
      return $this->gFileName;
   }
   
   public function getCachedImageName()
   {
      return $this->gCachedImageName;
   }
   
   public function getCachedThumbnailName()
   {
      return $this->gCachedThumbnailName;
   }
}
?>
