<?php
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/GedComException.php" );
    
class GedComNode
{
   
   
   
   private $gLineNumber = 0;
   
   protected $gLine;
   
   
   protected $gParentNode;
   
   
   protected $gLevelNumber;
   
   
   protected $gData;
   
   
   protected $gNodeList;
   
   
   protected $gIdentifier = "";
   
   
   protected $gTagName = "gedcom";
   
   
   
   
    
   public function __construct( $inLine )
   {
      if ( $inLine != NULL ) {
         $this->gData = rtrim( $inLine->getData() );
         $this->gTagName = $inLine->getTag();
         $this->gIdentifier = $inLine->getIdentifier();
         $this->gLevelNumber = $inLine->getLevel();
      } else {
         $this->gLevelNumber = 0;
      }
   }
   
   
   
   
 
   public function addNode( GedcomNode $inNode )
   {
      if ( $this->gNodeList == NULL ) {
         $this->gNodeList = array();
      }
      $this->gNodeList[] = $inNode;
      return $inNode;
   }
   
   
   
   public function parse( GedcomReader $inReader )
   {
      $nextNode = null;
      try {
         $nextLine = $inReader->getNextLine();
         $nextNode = new GedcomNode( $nextLine );
         $nextNode->gLineNumber = $nextLine->getLineNumber();
         while ( $nextNode->getLevel() != -1 ) {
            
            if ( $nextNode->getLevel() <= $this->gLevelNumber ) {
               
               return $nextNode;
            } else {
               
               $nextNode->gParentNode = $this;
               $this->addNode( $nextNode );
               if ( $nextNode->getTag() == "TRLR" ) {
                  return $nextNode;
               } else {
                  
                  $nextNode = $nextNode->parse( $inReader );
                  if ( $nextNode->getTag() == "TRLR" ) {
                     return $nextNode;
                  }
               }
            }
         }
      } catch ( GedComException $e ) {
         throw new GedComException( $e->getMessage(), $e->getCode );
      } 
   }
   
   
   
   
   public function setParentNode( $inNode )
   {
      $this->gParentNode = $inNode;
   }
   
   
   
   
   public function setLineNumber( $inLineNumber )
   {
      $this->gLineNumber = $inLineNumber;
   }
   
   
   
   
   public function getTag()
   {
      
      return $this->gTagName;
   }
   
   
   
   
   public function getLineNumber()
   {
      return $this->gLineNumber;
   }
   
   
   
   
   public function getLevel()
   {
      return $this->gLevelNumber;
   }
   
   
   
   
   public function getData()
   {
      return $this->gData;
   }
   
   
   
   
   public function toString()
   {
      $msg = $this->gLevelNumber . " " . $this->gTagName . " " . $this->gData . "\n";
      if ( !empty( $this->gNodeList ) ) {
         foreach ( $this->gNodeList as $node ) {
            $msg = $msg . $node->toString();
         }
      }
      return $msg;
   }
    
   
   
   public function toXMLString()
   {
      $msg = "";
      $attribName = "data";
      if ( $this->gTagName == "CHIL" 
         || $this->gTagName == "SUBM"
         || $this->gTagName == "FAMS"
         || $this->gTagName == "FAMC"
         || $this->gTagName == "HUSB"
         || $this->gTagName == "WIFE" ) {
         
         $attribName = "ref";
         $msg .= "<g:" . strtolower( $this->gTagName );
         if ( strlen( $this->gData ) > 0 ) {
            $msg .= " " . $attribName . "=\"" . rtrim( GedCom::convertString( $this->getData() ) ) . "\"";
         }
         $msg .= "/>\n";
      } else if ( $this->gTagName == "DATE"
            || $this->gTagName == "FORM"
            || $this->gTagName == "SURN"
            || $this->gTagName == "VERS"
            || $this->gTagName == "CHAR"
            || $this->gTagName == "NAME"
            || $this->gTagName == "EMAIL"
            || $this->gTagName == "WEB"
            || $this->gTagName == "CONT"
            || $this->gTagName == "TITL"
            || $this->gTagName == "OCCU"
            || $this->gTagName == "GIVN"
            || $this->gTagName == "SEX"
            || $this->gTagName == "PLAC"
            || $this->gTagName == "CONC"
            || $this->gTagName == "REFN"
            || $this->gTagName == "EDUC" ) {
         
         $msg .= "<g:" . strtolower( $this->gTagName );
         $attribName = "data";
         if ( strlen( $this->gData ) > 0 ) {
            $msg .= " " . $attribName . "=\"" . rtrim( GedCom::convertString( $this->getData() ) ) . "\"";
         }
         $msg .= "/>\n";
      } else if ( $this->gTagName == "QUAY") {
         
         $attribName = "level";
         $msg .= "<g:" . strtolower( $this->gTagName );
         if ( strlen( $this->gData ) > 0 ) {
            $msg .= " " . $attribName . "=\"" . rtrim( GedCom::convertString( $this->getData() ) ) . "\"";
         }
         $msg .= "/>\n";
      } else if ( $this->gData == "FAM"
            || $this->gData == "SOUR"
            || $this->gData == "INDI"
            || $this->gData == "SUBM" ) {
         
         $attribName = "id";
         $tagName = strtolower( $this->gData );
         $msg .= "<g:" . $tagName;
         if ( strlen( $this->gData ) > 0 ) {
            $msg .= " " . $attribName . "=\"" . $this->gIdentifier . "\"";
         }
         $msg .= ">\n";
         if ( !empty( $this->gNodeList ) ) {
            foreach( $this->gNodeList as $node ) {
               $msg .= $node->toXMLString();
            }
         }
         $msg .= "</g:" . $tagName . ">\n";
      } else if ( $this->gTagName == "gedcom" ) {
         
         
         $msg .= "<gedcom xmlns:g=\"http://gedcom.org/dtd/gedxml55.dtd\">\n";
         if ( !empty( $this->gNodeList ) ) {
            foreach( $this->gNodeList as $node ) {
               $msg .= $node->toXMLString();
            }
         }
         $msg .= "</gedcom>\n";
      } else {
         
         $msg .= "<g:" . strtolower( $this->gTagName );
         if ( strlen( $this->gData ) > 0 ) {
            $msg .= " " . $attribName . "=\"" . rtrim( GedCom::convertString( $this->getData() ) ) . "\"";
         }
         $msg .= ">\n";
         if ( !empty( $this->gNodeList ) ) {
            foreach( $this->gNodeList as $node ) {
               $msg .= $node->toXMLString();
            }
         }
         $msg .= "</g:" . strtolower( $this->gTagName ) . ">\n";
      }
      return $msg;
   }
}
?>
