<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/caching/CacheFile.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/generation/GeneratorPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
    
class PXTemplateGenerator extends GeneratorPipeElement implements PipeElementInterface
{
    
   
   
   
    
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      parent::run( $inVariableStack, $inURL, $inQueryString, $inDOM );
      
      $fullFileName = StringResolver::expandString( $inVariableStack, $this->gSrc );
      $fullFileName = StringResolver::expandPseudoProtocols( $fullFileName );
      $cacheKey = CacheFile::getHash( $fullFileName );
      $isCachable = $this->gComponent->gIsCachable;
      $useReqParams = Utilities::normalizeBoolean( $this->gComponent->getUseRequestParametersFlag() );
      
      
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $localCachable = $requestParameterModule->get( CacheFile::CACHABLE_ATTRIBUTE_NAME );
      if ( $useReqParams && $localCachable != NULL ) {
         $localCachable = Utilities::normalizeBoolean( $localCachable );
         $isCachable = $localCachable;
      }
      if ( $isCachable ) {
         $this->gCacheFile = new CacheFile( Environment::$configuration[ 'cacheDir' ] );
         $oldParametersHash = $this->gCacheFile->getParameterHash( $cacheKey );
         $newParameterHash = CacheFile::getHash( $this->gParameters->toString() );
         $validParameters = ( $oldParametersHash == $newParameterHash );
         
         
         $oldSrcTimeStamp = $this->gCacheFile->getTimeStamp( $cacheKey );
      } else {
         $oldParametersHash = 0;
         $validParameters = false;
         $oldSrcTimeStamp = 0;      
      }
      $documentDOM = new DomDocument();
      
      try {
         
         if ( file_exists( $fullFileName ) ) {
            $srcTimeStamp = filemtime( $fullFileName );
            $validSrcFile = ( $oldSrcTimeStamp == $srcTimeStamp );
            if ( $isCachable && $validSrcFile && $validParameters ) {
               
               $documentDOM = $this->gCacheFile->getCacheDOM( $cacheKey );
               $documentDOM->documentElement->setAttribute( "__file", $cacheKey );
               $documentDOM->documentElement->setAttribute( "__status", CacheFile::CACHE_VALID );
            } else {
               
               if ( !($fileText = file_get_contents( $fullFileName ) ) ) {
                  throw new RunException( "Could not load document string from file '$fullFileName'",
                     PalooseException::SITEMAP_GENERATOR_RUN_ERROR );
               }
               $documentDOM->loadXML( Utilities::xlateEntity( $fileText ) );
               $rootNode = $documentDOM->documentElement;
               $this->gTranslatedDOM = new DOMDocument();
               $elementNS = $rootNode->namespaceURI;
               $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
               $this->gTranslatedDOM->appendChild( $newElement );
               $this->processNode( $inVariableStack, $rootNode, $newElement );
               
               $documentDOM = $this->gTranslatedDOM;
               if ( $isCachable ) {
                  $this->gCacheFile->updateDOM2Cache( $cacheKey, $srcTimeStamp, $documentDOM, $newParameterHash );
               }
               
               
               $documentDOM->documentElement->setAttribute( "__file", $cacheKey );
               
               $documentDOM->documentElement->setAttribute( "__status", CacheFile::CACHE_INVALID );
               
           }
         } else {
              throw new RunException( "File '$fullFileName' does not exist", PalooseException::SITEMAP_GENERATOR_RUN_ERROR );
         }
      } catch ( RunException $e ) {
           throw new RunException( $e->getMessage(), $e->getCode() );
      }
      return $documentDOM;
   }
   
   
   
   private function processNode( VariableStack $inVariableStack, DOMNode $inParentNode, DOMNode &$inNewParentNode )
   {
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $attribute->value ) );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         }
      }
      
      
      $children = $inParentNode->childNodes;
      foreach ( $children as $child ) {
         if ( $child->nodeType == XML_ELEMENT_NODE ) {
           
           $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
           
           $this->processNode( $inVariableStack, $child, $newElementNode );
           $inNewParentNode->appendChild( $newElementNode );
         } else if ( $child->nodeType == XML_TEXT_NODE ) {
            
            
            $expandedContent = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $child->textContent ) );
            $newTextNode = $this->gTranslatedDOM->createTextNode( $expandedContent );
            
            $inNewParentNode->appendChild( $newTextNode );
         }
      }
   }
   
   
   
   
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
}
 
class _PXTemplateGenerator extends Generater
{
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "PXTemplateGenerator";
      
   }
}
?>
