<?php
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Aggregator.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Mount.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Redirect.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/Pipe.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/selection/Selectors.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
class SimpleMatch {
      
   private $gPipe;
      
   private $gPattern;
      
   private $gType;
      
   private $gDefaultMatcher;
      
   private $gMatcher;
      
   
   private $gXPath;
   
   
   public function __construct()
   {
   }
   
   
   
   public function parse( $inNode ) 
   {
      
      $matchers = Environment::$sitemapStack->root()->getMatchers();
      $this->gDefaultMatcher = $matchers->getDefault();
      
      $this->gPattern = $inNode->getAttribute( "pattern" );
      $this->gType = $inNode->getAttribute( "type" );
      if ( !$this->gType ) {
         $this->gType = $this->gDefaultMatcher;
      }
      
      try {
         $this->gMatcher = $matchers->getMatcherClassName( $this->gType );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
      
      $matchDom = new DOMDocument;
      $matchDom->appendChild( $matchDom->importNode( $inNode, 1 ) );
      
      if ( $this->gPattern == "" ) {
         throw new UserException(
            "Must be a pattern attribute present and not NULL",
            PalooseException::SITEMAP_MATCH_PARSE_ERROR,
            $matchDom );
      }
      $this->gXPath = new domxpath( $matchDom );
      $this->gXPath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      $component = $this->gXPath->query( "//m:match/*" );
      $this->gPipe = new Pipe();
      foreach ( $component as $pipeNode ) {
         $name = $pipeNode->localName;
         
         switch ( $name ) {
               case "aggregate" :
                  try {
                     $aggregator = PipeElement::createAggregator( $pipeNode );
                     
                     $this->gPipe->addComponent( $aggregator );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               
               case "generate" :
                  try {
                     $generator = PipeElement::createGenerator( $pipeNode );
                     $this->gPipe->addComponent( $generator );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
                  
               case "transform" :
                  try {
                     $transformer = PipeElement::createTransformer( $pipeNode );
                     $this->gPipe->addComponent( $transformer );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               case "serialize" :
                  try {
                     $serializer = PipeElement::createSerializer( $pipeNode );
                     $this->gPipe->addComponent( $serializer );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
               
               case "read" :
                  try {
                     $reader = PipeElement::createReader( $pipeNode );
                     $this->gPipe->addComponent( $reader );
                  } catch ( UserException $e ) {
                     throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
                  }
                  break;
            case "mount" :
               try {
                  $mount = PipeElement::createMount( $pipeNode );
                  $this->gPipe->addComponent( $mount );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "select" :
               try {
                  $selector = PipeElement::createSelector( $pipeNode );
                  $this->gPipe->addComponent( $selector );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "call" :
               try {
                  $call = PipeElement::createCall( $pipeNode );
                  
                  $this->gPipe->addComponent( $call );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
               
            case "act" :
               try {
                  $action = PipeElement::createAction( $pipeNode );
                  $this->gPipe->addComponent( $action );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
            case "redirect-to" :
               try {
                  $redirect = PipeElement::createRedirect( $pipeNode );
                  $this->gPipe->addComponent( $redirect );
               } catch ( UserException $e ) {
                  throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
               }
               break;
}
         
      }
   }
   
   
   
   
   public function run( $inURL )
   {
      
      $matcher = new $this->gMatcher();
      $map = array();
      try {
         $isMatch = $matcher->match( $inURL, $this->gPattern );
      } catch ( MatcherException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode() );
      }
      $map = $matcher->getHashMap();
      try {
         if ( $isMatch == 1 ) {
            for ( $i = 0; $i < count( $map ); $i++ ) {
               
            }
            
            Environment::$variableStack->push( $map );
            
            $this->gPipe->run( Environment::$variableStack, $inURL, Environment::$gQueryString, NULL );
            
            
            Environment::$variableStack->pop();
            return true;
         } else {
            
         }
      } catch ( ExitException $e ) {
         throw new ExitException();
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
      return false;
   } 
   
   
   public function getMatcher()
   {
      return $this->gMatcher;
   }
   
   
   
   
   public function toString()
   {
      $mess = "   <match pattern='{$this->gPattern}' type='{$this->gType}' package='{$this->gMatcher}'>\n";
      $mess .= $this->gPipe->toString();
      $mess .= "   </match>\n";
      return $mess;
   } 
}
?>
