<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/reading/Reader.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
class Readers {
   private $gDefault;
   
   private $gReaders;
      
   
   
   
   public function __construct()
   {
   }
   
   
   
   public function parse( $inNode )
   {
      
      $readersDom = new DOMDocument;
      $readersDom->appendChild( $readersDom->importNode( $inNode, 1 ) );
      
   
      
      $xpath = new domxpath( $readersDom );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $component = $xpath->query( "//m:reader" );
      
      foreach ( $component as $node ) { 
         $name = $node->getAttribute( "name" );
         $src = $node->getAttribute( "src" );
         $isCachable = Utilities::normalizeBoolean( $node->getAttribute( "cachable" ) );
         if ( !strlen( $name ) or !strlen( $src ) ) {
            $dom = new DOMDocument;
            $dom->appendChild( $dom->importNode( $node, 1 ) );
            throw new UserException( "Component Reader must have name and src attributes defined", PalooseException::SITEMAP_READER_PARSE_ERROR, $dom );
         }
         
         
         if ( strstr( $src, Environment::getPathSeparator() ) == 1 ) {
            $packageFilename = $src;
         } else if ( !strstr( $src, ":" . Environment::getPathSeparator() ) ) {
            
            $packageFilename = PALOOSE_LIB_DIRECTORY . Environment::getPathSeparator() . $src;
         } else {
            
            $packageFilename = StringResolver::expandPseudoProtocols( $src );
         }
         $package = Utilities::stripFileName( $packageFilename );
         
         $path = Utilities::stripPath( StringResolver::expandPseudoProtocols( $packageFilename ) );
         
         
         if ( $this->gReaders and array_key_exists( $name, $this->gReaders ) ) {
         } else {
         }
            
         
         require_once( $path . $package . ".php" );
         
         $readerName = "_" . $package;
         $reader = new $readerName( $name, $path.$package, $node, $isCachable );
         
         try {
            $reader->isValid();
            $this->gReaders[ $name ] = $reader;
         } catch( UserException $e ) {
            $readerDom = new DOMDocument;
            $readerDom->appendChild( $readerDom->importNode( $node, 1 ) );
            throw new UserException( $e->getMessage(), $e->getCode(), $readerDom );
         }
      }
   }
   
   
   
   public function setDefault( $inDefault )
   {
      if ( !( array_key_exists( $inDefault, $this->gReaders ) ) ) {
           throw new UserException( "Default Reader '". $inDefault . "' not found", PalooseException::SITEMAP_READER_PARSE_ERROR );
      }
      $this->gDefault = $inDefault;
   }
   
   
   
   public function getDefault()
   {
      return $this->gDefault;
   }
   
   
   
   public function getReader( $inName )
   {
      if ( !( array_key_exists( $inName, $this->gReaders ) ) ) {
           throw new InternalException( "Reader '". $inName . "' not found", PalooseException::SITEMAP_READER_PARSE_ERROR );
      } else {
         return $this->gReaders[ $inName ];
      }
   }
   
   
   
   public function getReaderClassName( $inName )
   {
      if ( !( array_key_exists( $inName, $this->gReaders ) ) ) {
           throw new InternalException( "Reader '". $inName . "' not found", PalooseException::SITEMAP_READER_PARSE_ERROR );
      } else {
         $className = $this->gReaders[ $inName ]->getPackageName();
         return $className;
      }
   }
   
   
   
   public function toString()
   {
      $mess = "  <readers default='{$this->gDefault}'>\n";
      foreach ( $this->gReaders as $node ) {
         $mess .= $node->toString();
      }
      $mess .= "  </readers>\n";
      return $mess;
   }
}
?>
