<?php
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Utilities.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/environment/StringResolver.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
class Redirect extends PipeElement implements PipeElementInterface
{
    
   private $gURI;
    
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      $this->gGenericType = PipeElement::REDIRECT;
      $this->gSitemapSrc = $inSrc;
      $this->gType = $inType;
      $this->gComponent = $inComponent;
      
      $xpath = new domxpath( $inDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $this->gURI = Utilities::getXPathListStringItem( 0, $xpath, "//@uri" );
       if ( $this->gURI == NULL or strlen( $this->gURI ) == 0 ) {
         throw new UserException(
            "Must have a uri attribute defined",
            PalooseException::SITEMAP_REDIRECT_PARSE_ERROR,
            $inDOM );
       }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      $expandedURI = StringResolver::removePseudoProtocols( $this->gURI );
      $expandedURI = StringResolver::expandString( $inVariableStack, $expandedURI );
      
      $urlComponents = parse_url( $expandedURI );
      if ( $urlComponents === false ) {
         throw new UserException( "Malformed URI '". $expandedURI . "' in redirect", PalooseException::SITEMAP_REDIRECT_PARSE_ERROR );
      } else if ( array_key_exists( "scheme", $urlComponents ) and ( $urlComponents[ 'scheme' ] == "http" or $urlComponents[ 'scheme' ] == "https" ) ) {
        
        ob_end_clean();
         header( "Location: {$expandedURI}" );
         throw new ExitException();
      }
        
     
      $uriSource = Environment::getSource( $this->gURI );
      
      if ( $uriSource == Environment::UNKNOWN_SOURCE ) $uriSource = Environment::SUBMIT_ROOT ;
      switch ( $uriSource ) {
         case Environment::SUBMIT_ROOT :
            
            $sitemap = Environment::$sitemapStack->root();
            $sitemap->run( $expandedURI, $inQueryString, true  );  
            break;
         case Environment::SUBMIT_CURRENT :
            
            $sitemap = Environment::$sitemapStack->peek();
            $sitemap->run( $expandedURI, $inQueryString, true  );  
            break;
         default :
            throw new RunException(
               "Illegal return code '$uriSource' for string expansion source processing '$this->gURI'",
               PalooseException::UNKNOWN_STRING_EXPANSION_TYPE );
      }
    
   }
   
   
   
   public function getURI()
   {
      return $this->gURI;
   }
   
   
   
   public function toString()
   {
      $mess = "    <redirect uri='{$this->gURI}'/>\n";
      return $mess;
   }
}
?>
