<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/generation/Generators.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/Transformers.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/serialization/Serializers.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/reading/Readers.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/flows/Flow.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/serialization/OutputStream.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/Pipelines.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/matching/Matchers.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/selection/Selectors.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/acting/Actions.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/viewing/Views.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/resources/Resources.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/InternalException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/UserException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/RunException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/exception-handling/ExitException.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
 
class Sitemap {
   
   private $gSitemapFileName;
   
   
   private $gSitemapPathName;
   
   
   public $gGenerators;
   
   
   public $gTransformers;
   
   
   public $gSerializers;
   
   
   public $gReaders;
   
   
   public $gSelectors;
   
   
   public $gPipelines;
   
   public $gMatchers;
   
   public $gViews;
   
   public $gResources;
   
   public $gActions;
   
   public $gFlow;
   
   private $gXPath;
   
   private $gOutputStream;
      
   
   
   
   public function __construct( $inSitemapPathName, $inSitemapFileName, $inSitemapStack, $inOutputStream )
   {
      $this->gSitemapFileName = $inSitemapPathName . Environment::getPathSeparator() . $inSitemapFileName;
      $this->gOutputStream = $inOutputStream;
      $this->gSitemapPathName = $inSitemapPathName;
      
      Environment::$gCurrentSitemapDir = $inSitemapPathName;
      if ( !file_exists( $this->gSitemapFileName ) ) {
         
         throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not exist.", PalooseException::MISSING_SITEMAP, NULL );
      }
      if ( $inSitemapStack == NULL ) {
         
         $this->gGenerators = new Generators();
         $this->gTransformers = new Transformers();
         $this->gSerializers = new Serializers();
         $this->gReaders = new Readers();
         $this->gMatchers = new Matchers();
         $this->gPipelines = new Pipelines();
         $this->gSelectors = new Selectors();
         $this->gActions = new Actions();
         $this->gViews = new Views();
         $this->gResources = new Resources();
         $this->gFlow = new Flow();
      } else {
         
         
         $callingSitemap = $inSitemapStack->peek();
         $this->gGenerators = $callingSitemap->getGenerators();
         $this->gTransformers = $callingSitemap->getTransformers();
         $this->gSerializers = $callingSitemap->getSerializers();
         $this->gReaders = $callingSitemap->getReaders();
         $this->gMatchers = $callingSitemap->getMatchers();
         $this->gPipelines = $callingSitemap->getPipelines();
         $this->gSelectors = $callingSitemap->getSelectors();
         $this->gActions = $callingSitemap->getActions();
         $this->gViews = new Views();               
         $this->gResources = new Resources();       
         $this->gFlow = new Flow();
      }
    }
   
   
   
   public function parse()
   {
      $this->SitemapDOM = new DOMDocument();
      
      $check = new XML_check();
      if ( !$check->check_url( $this->gSitemapFileName ) ) {
         throw new UserException( "Sitemap error '{$this->gSitemapFileName}'" .
            "\n" . $check->get_full_error(),
            PalooseException::SITEMAP_PARSE_ERROR );
      }
      if ( $this->SitemapDOM->load( $this->gSitemapFileName ) === false ) {
         throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse.", PalooseException::SITEMAP_PARSE_ERROR );
      }
      
      $this->gXPath = new domxpath( $this->SitemapDOM );
      $this->gXPath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      
      
      try {
         $this->parseComponents();
         $this->parsePipelines();
      } catch( UserException $e ) {
         throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   
   private function parseComponents()
   {
      try {
         $this->parseGenerators();
         $this->parseTransformers();
         $this->parseSerializers();
         $this->parseReaders();
         $this->parseMatchers();
         $this->parseViews();
         $this->parseSelectors();
         $this->parseActions();
         $this->parseResources();
         $this->parseFlow();
      } catch( UserException $e ) {
         throw new UserException( $e->getmessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   
   public function setOutputStream( $inOutStream )
   {
      $this->gOutputStream = $inOutStream;
   }
   
   
   public function getOutputStream()
   {
      return $this->gOutputStream;
   }
   
   
   
   public function getSitemapStack()
   {
      return $this->gSitemapStack;
   }
   
   
   
   private function parseGenerators()
   {
      $component = $this->gXPath->query( "//m:components/m:generators" );
      
      
      
      foreach ( $component as $node ) { 
         try {
            $this->gGenerators->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in generator declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      } 
      
      
      $root = $this->gXPath->query( "//m:components/m:generators" );
      
      foreach ( $root as $node ) {
         $default = $node->getAttribute( "default" );
         try {
            $this->gGenerators->setDefault( $default );
         } catch ( UserException $e ) {
            
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in generator declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      } 
   } 
   
   
   
   public function getGenerators()
   {
      return $this->gGenerators;
   } 
   
   
   
   private function parseTransformers()
   {
      $component = $this->gXPath->query( "//m:components/m:transformers" );
      
      
      foreach ( $component as $node ) { 
      
         try {
            $this->gTransformers->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in transformer declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
      
      
      $root = $this->gXPath->query( "//m:components/m:transformers" );
      
      foreach ( $root as $node ) {
         $default = $node->getAttribute( "default" );
         try {
            $this->gTransformers->setDefault( $default );
         } catch ( UserException $e ) {
            
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in transformer declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   
    
   public function getTransformers()
   {
      return $this->gTransformers;
   } 
   
   
   
   private function parseSerializers()
   {
      $component = $this->gXPath->query( "//m:components/m:serializers" );
      foreach ( $component as $node ) { 
         try {
            $this->gSerializers->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in serializer declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
      
      $root = $this->gXPath->query( "//m:components/m:serializers" );
      
      foreach ( $root as $node ) {
         $default = $node->getAttribute( "default" );
         try {
            $this->gSerializers->setDefault( $default );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in serializer declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   
    
   public function getSerializers()
   {
      return $this->gSerializers;
   } 
    
   
   
   private function parseReaders()
   {
      $component = $this->gXPath->query( "//m:components/m:readers" );
      
      
      foreach ( $component as $node ) { 
         try {
            $this->gReaders->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in readers declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
      
      
      $root = $this->gXPath->query( "//m:components/m:readers" );
      
      foreach ( $root as $node ) {
         $default = $node->getAttribute( "default" );
         try {
            $this->gReaders->setDefault( $default );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in readers declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   
    
   public function getReaders()
   {
      return $this->gReaders;
   } 
   
   
   
   private function parseSelectors()
   {
      $component = $this->gXPath->query( "//m:components/m:selectors" );
      
      
      foreach ( $component as $node ) { 
         try {
            $this->gSelectors->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in selectors declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
      
      
      $root = $this->gXPath->query( "//m:components/m:selectors" );
      
      foreach ( $root as $node ) {
         $default = $node->getAttribute( "default" );
         try {
            $this->gSelectors->setDefault( $default );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in selectors declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   public function getSelectors()
   {
      return $this->gSelectors;
   } 
   
   
   
   private function parseMatchers()
   {
      $component = $this->gXPath->query( "//m:components/m:matchers" );
      foreach ( $component as $node ) { 
         try {
            $this->gMatchers->parse( $node );
         } catch ( UserException $e ) {
            
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in matchers declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
      
      $root = $this->gXPath->query( "//m:components/m:matchers" );
      
      foreach ( $root as $node ) {
         $default = $node->getAttribute( "default" );
         try {
            $this->gMatchers->setDefault( $default );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in matchers declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   public function getMatchers()
   {
      return $this->gMatchers;
   } 
   
   
   
   private function parseViews()
   {
      $component = $this->gXPath->query( "//m:views" );
      foreach ( $component as $node ) { 
         try {
            $this->gViews->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in views declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   public function getViews()
   {
      return $this->gViews;
   } 
   
   
   
   private function parseResources()
   {
      $resources = $this->gXPath->query( "//m:resources" );
      foreach ( $resources as $node ) { 
         try {
            $this->gResources->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in resources declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   
   public function getResources()
   {
      return $this->gResources;
   } 
   
   
   
   private function parsePipelines()
   {
      $component = $this->gXPath->query( "//m:pipelines" );
      
      foreach ( $component as $node ) { 
         try {
            $this->gPipelines->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in pipelines declaration :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
 
   
   
   
   public function getPipelines()
   {
      return $this->gPipelines;
   } 
   
   
   
   private function parseActions()
   {
      $component = $this->gXPath->query( "//m:components/m:actions" );
      
      foreach ( $component as $node ) { 
         try {
            $this->gActions->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in actions declaration" . " :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
      
   } 
   
   
   
   public function getActions()
   {
      return $this->gActions;
   } 
   
   
   
   private function parseFlow()
   {
      $component = $this->gXPath->query( "//m:flow" );
      
      foreach ( $component as $node ) { 
         try {
            $this->gFlow->parse( $node );
         } catch ( UserException $e ) {
            throw new UserException( "Sitemap '" . $this->gSitemapFileName . "' does not parse in flow declaration" . " :\n" . $e->getMessage(),
               $e->getCode(),
               $e->getDOMScrap() );
         }
      }
   } 
   
   
   
   public function run( $inURL, $inQueryString, $inInternalRequest )
   {
      Environment::$gCurrentSitemapDir = $this->gSitemapPathName;
      
      
      Environment::$sitemapStack->push( $this );
      
      try {
         $success = $this->gPipelines->run( $inURL, $inQueryString, $inInternalRequest );
         if ( $success === false ) {
            throw new RunException( "URL not found: " . $inURL, "404" );
         }
      } catch ( ExitException $e ) {
         throw new ExitException();
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode() );
      }
      
      Environment::$sitemapStack->pop();
   } 
   
   
   
   
   public function getFilename()
   {
      return $this->gSitemapFileName;
   }
 
   
   
   
    
   public function toString()
   {
      $m = "<sitemap>\n";
      $m .= " <components>\n";
      $m .= $this->gGenerators->toString();
      $m .= $this->gTransformers->toString();
      $m .= $this->gSerializers->toString();
      $m .= $this->gReaders->toString();
      $m .= $this->gMatchers->toString();
      $m .= $this->gSelectors->toString();
      $m .= $this->gActions->toString();
      $m .= $this->gFlow->toString();
      $m .= " </components>\n";
      $m .= $this->gResources->toString();
      $m .= $this->gViews->toString();
      $m .= $this->gPipelines->toString();
      $m .= "</sitemap>\n";
      return $m;
   } 
 
}
?>
