<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/i18n/Catalogues.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
class I18nTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
     
   private $gId;
    
   private $gLocale;
    
   private $gDOM;
    
   private $gTranslationPossible;
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
        try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      $this->gDOM = $inDOM;
     
      
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $params = $requestParameterModule->getParams();
      if ( !empty( $params ) ) {
         foreach ( $params as $name => $value ) {
            $this->gParameters->setParameter( $name, $value );
         }
      }
      $this->gLocale = $this->gParameters->getParameter( "locale" );
      if ( $this->gLocale == "" ) {
         $this->gLocale = "en_GB";
         
         
         
      }
      
      $catalogues = $this->gComponent->getCatalogues();
      $catalogueName = $catalogues->getDefaultCatalogue();
      if ( !empty( $this->gParameters->getParameter( "default-catalogue-id" ) ) ) {
         $catalogueName = $this->gParameters->getParameter( "default-catalogue-id" );
      }
      $expandedValue = StringResolver::expandString( $inVariableStack, $catalogueName );
      $catalogueName = StringResolver::expandPseudoProtocols( $expandedValue );
      
      
      try {
         $catalogue = $catalogues->getCatalogue( $catalogueName );
         
         $location = $catalogue->getLocation();
         $id = $catalogue->getId();
         $name = $catalogue->getName();
         $untranslatedText = $this->gComponent->getUntranslatedText();
   
         
         $expandedValue = StringResolver::expandString( $inVariableStack, $location );
         $expandedValue = StringResolver::expandPseudoProtocols( $expandedValue );
                
         
         $languageCountryFileName = $expandedValue . Environment::getPathSeparator() . $name . "_" . $this->gLocale . ".xml";
         if ( strstr( $this->gLocale, "_" ) !== false ) {
            $pieces = explode( "_", $this->gLocale );
            $country = "_" . $pieces[ 0 ];
         } else {
            $country = "";
         }
         $languageFileName = $expandedValue . Environment::getPathSeparator() . $name . $country . ".xml";
         $neutralFileName = $expandedValue . Environment::getPathSeparator() . $name . ".xml";
         $this->gTranslationPossible = true;
         if ( file_exists( $languageCountryFileName ) ) {
            $fullFileName = $languageCountryFileName;
         } else if ( file_exists( $languageFileName ) ) {
            $fullFileName = $languageFileName;
         } else if ( file_exists( $neutralFileName ) ) {
            $fullFileName = $neutralFileName;
         } else {
            
            $this->gTranslationPossible = false;
         }
      } catch ( RunException $e ) {
         $this->gTranslationPossible = false;
      }
      if ( $this->gTranslationPossible ) {
         
         
         $catalogueDOM = new DOMDocument();
         if ( !$catalogueDOM->load( $fullFileName ) ) {
            throw new UserException( "Could not load document from translation file '$fullFileName'", PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
         }
   
         
         
         
         $rootNode = $inDOM->documentElement;
   
         $this->gTranslatedDOM = new DOMDocument();
         $elementNS = $rootNode->namespaceURI;
         $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
         $this->gTranslatedDOM->appendChild( $newElement );
         
         $this->processNode( $catalogueDOM, $rootNode, $newElement );
         return $this->gTranslatedDOM;
      } else {
         
         $rootNode = $inDOM->documentElement;
   
         $this->gTranslatedDOM = new DOMDocument();
         $elementNS = $rootNode->namespaceURI;
         $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
         $this->gTranslatedDOM->appendChild( $newElement );
         
         $this->processNode( null, $rootNode, $newElement );
         return $this->gTranslatedDOM;
      }
   }
   
   
   
   private function processNode( $inCatalogueDOM, DOMNode $inParentNode, DOMNode $inNewParentNode )
   {
      
      
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         $i18nAttribs = array();
         $i = 0;
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               
               $attribName = $attribute->name;
               $attribValue = $attribute->value;
               if ( $attribute->namespaceURI == Environment::$configuration[ 'i18nNamespace' ] and $attribName == "attr" ) {
                  $i18nAttribs[ $i++ ] = $attribValue;
               }
            }
         }
         
         
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               
               $attribName = $attribute->name;
               $attribValue = $attribute->value;
               if ( in_array( $attribName, $i18nAttribs ) and $this->gTranslationPossible === true ) {
                  
                  if ( $inCatalogueDOM != null ) {
                     $attribValue = $this->getAttributeTranslation( $inCatalogueDOM, $attribValue );
                  } else {
                     $attribValue = $this->gComponent->getUntranslatedText();
                  }
               }
               
               if ( $attribute->namespaceURI !== Environment::$configuration[ 'i18nNamespace' ] ) {
                  $inNewParentNode->setAttribute( $attribName, $attribValue );
               }
            }
         } else {
            
         }
      } 
      
      
      $children = $inParentNode->childNodes;
      
      foreach ( $children as $child ) {
         if ( $child->nodeType == XML_ELEMENT_NODE ) {
            if ( $child->namespaceURI == Environment::$configuration[ 'i18nNamespace' ] ) {
               
               
               $nodeName = $child->localName;
               
               if ( $nodeName == "text" ) {
                  $this->processText( $inCatalogueDOM, $child, $inNewParentNode );
               } else if ( $nodeName == "date" ) {
                  $this->processDate( $child, $inNewParentNode );
               } else if ( $nodeName == "time" ) {
                  $this->processTime( $child, $inNewParentNode );
               }
               
           } else {
              
              $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
              
              $this->processNode( $inCatalogueDOM, $child, $newElementNode );
              $inNewParentNode->appendChild( $newElementNode );
           }
         } else if ( $child->nodeType == XML_TEXT_NODE ) {
            
            $newTextNode = $this->gTranslatedDOM->createTextNode( $child->textContent );
            $inNewParentNode->appendChild( $newTextNode );
         }
      }
   }
   
   
   
   private function processTime( DOMNode $inChildNode, DOMNode $inNewParentNode )
   {
      if ( $inChildNode->hasAttribute( "value" ) ) {
         $timeValue = strtotime( $inChildNode->getAttribute( "value" ) );
      } else if ( $inChildNode->textContent == "" ) {
         $timeValue = strtotime( "now" );
      } else {
         
         $timeValue = strtotime( $inChildNode->textContent );
      }
      if ( $inChildNode->hasAttribute( "pattern" ) ) {
         $timeFormat = $inChildNode->getAttribute( "pattern" );
         
      } else {
         $timeFormat = "%H:%M:%S";      
      }
      setlocale( LC_ALL, $this->gLocale );
      $newTextNode = $this->gTranslatedDOM->createTextNode( strftime( $timeFormat, $timeValue ) );
      $inNewParentNode->appendChild( $newTextNode );
   }
   
   
   
   private function processDate( DOMNode $inChildNode, DOMNode $inNewParentNode )
   {
      if ( $inChildNode->hasAttribute( "value" ) ) {
         $dateValue = strtotime( $inChildNode->getAttribute( "value" ) );
      } else if ( $inChildNode->textContent == "" ) {
         $dateValue = strtotime( "now" );
      } else {
         
         $dateValue = strtotime( $inChildNode->textContent );
      }
     if ( $inChildNode->hasAttribute( "pattern" ) ) {
         $dateFormat = $inChildNode->getAttribute( "pattern" );
         
      } else {
         $dateFormat = "%d %b %Y";      
      }
      setlocale( LC_ALL, $this->gLocale );
      $newTextNode = $this->gTranslatedDOM->createTextNode( strftime( $dateFormat, $dateValue ) );
      $inNewParentNode->appendChild( $newTextNode );
   }
   
   
   
   private function processText( $inCatalogueDOM, DOMNode $inChildNode, DOMNode $inNewParentNode )
   {
      if ( $this->gTranslationPossible === true and $inCatalogueDOM != null ) {
         
         if ( $inChildNode->hasAttribute( "key" ) ) {
            
            $keyValue = $inChildNode->getAttribute( "key" );
         } else {
            
            $keyValue = $inChildNode->textContent;
         }
         $xpath = new domxpath( $inCatalogueDOM );
         $translatedContent = $xpath->query( "//message[ @key = '$keyValue' ]" );
         
         $i = 0;
         if ( $translatedContent->length > 0 ) {
            while ( $i < $translatedContent->length ) {
               $node = $this->gTranslatedDOM->importNode( $translatedContent->item( $i ), true );
               if ( $node->nodeType == XML_ELEMENT_NODE ) {
               } else if ( $node->nodeType == XML_TEXT_NODE ) {
               }
               $inNewParentNode->appendChild( $node );
               $i++;
            }
         } else {
            $newTextNode = $this->gTranslatedDOM->createTextNode( $this->gComponent->getUntranslatedText() );
            $inNewParentNode->appendChild( $newTextNode );
         }
      } else {
         $node = $this->gTranslatedDOM->importNode( $inChildNode, true );
         $inNewParentNode->appendChild( $node );
      }
   }
   
   
   private function getAttributeTranslation( DOMDocument $inCatalogueDOM, $inValue )
   {
      $xpath = new domxpath( $inCatalogueDOM );
      $newValue = Utilities::getXPathListStringItem( 0, $xpath, "/message[ @key = '$inValue' ]/descendant::node()" );
      return $newValue;
   }
   
   
   
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
}
 
class _I18nTransformer extends Transformer {
    
   
    
   private $gUntranslatedText;
   
    
   private $gCatalogues;
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "I18nTransformer";
      $parameterDOM = new DOMDocument;
      $parameterDOM->appendChild( $parameterDOM->importNode( $inParameterNode, 1 ) );
      $xpath = new domxpath( $parameterDOM );
      $xpath->registerNamespace( "m", Environment::$configuration[ 'sitemapNamespace' ] );
      $catalogues = $xpath->query( "//m:catalogues" );
      $untranslatedNode = $parameterDOM->getElementsByTagName( "untranslated-text" );
      $this->gUntranslatedText = $untranslatedNode->item( 0 )->textContent;
      try {
         $this->gCatalogues = new Catalogues( $catalogues->item( 0 ) );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
   }
   
   
   public function getCatalogues()
   {
      return $this->gCatalogues;
   }
   
   
   public function getUntranslatedText()
   {
      return $this->gUntranslatedText;
   }
   
   
   
   public function toString()
   {
      if ( $this->gCatalogues != NULL ) {
         $mess = "   <transformer name='{$this->gName}' src='{$this->gSrc}' package='{$this->gPackageName}'>\n";
           $mess .= $this->gCatalogues->toString();
         $mess .= "   </transformer>\n";
      } else {
         $mess = "   <transformer name='{$this->gName}' src='{$this->gSrc}' package='{$this->gPackageName}'/>\n";
      }
      return $mess;
   }
 }
?>
