<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/caching/CacheFile.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
 
class PHPTransformer extends TransformerPipeElement implements PipeElementInterface
{
      
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
       
      if ( strlen( $inSrc ) == 0 ) {
         throw new UserException( "Must have a src attribute present", PalooseException::SITEMAP_TRANSFORMER_PARSE_ERROR, $inDOM );
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      parent::run( $inVariableStack, $inURL, $inQueryString, $inDOM );
      $documentDOM = $inDOM;
      
      $fullScriptFileName = StringResolver::expandString( $inVariableStack, $this->gSrc );
      $fullScriptFileName = StringResolver::expandPseudoProtocols( $fullScriptFileName );
      $validDOM = false;
      $validXSLFile = false;
      $validParameters = false;
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $isCachable = $this->gComponent->gIsCachable;
      $useReqParams = Utilities::normalizeBoolean( $this->gComponent->getUseRequestParametersFlag() );
      
      
      $localCachable = $requestParameterModule->get( CacheFile::CACHABLE_ATTRIBUTE_NAME );
      if ( $useReqParams && $localCachable != NULL ) {
         $localCachable = Utilities::normalizeBoolean( $localCachable );
         $isCachable = $localCachable;
      }
      $fullFileName = "";
      try {
         if ( !file_exists( $fullScriptFileName ) ) {
              throw new UserException(
                 "Cannot find script file '$fullScriptFileName'",
                 PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
         }
         
         require_once( $fullScriptFileName );
   
         $script = new PHPScript( $inVariableStack, $inURL, $inQueryString );
  
         $documentDOM = $script->run( $documentDOM, $this->gParameters->getParameterList() );
  
      } catch ( RunException $e ) {
           throw new RunException( $e->getMessage(), $e->getCode() );
      }
      if ( $documentDOM->documentElement ) {
         $documentDOM->documentElement->setAttribute( "__file", $fullFileName );
      }
      
      return $documentDOM;
   }
   
   
   
  
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
}
 
class _PHPTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "PHPTransformer";
   }
}
?>
