<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/caching/CacheFile.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
 
class TRAXTransformer extends TransformerPipeElement implements PipeElementInterface
{
      
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
      try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
      } catch ( UserException $e ) {
         throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
      }
       
      if ( strlen( $inSrc ) == 0 ) {
         throw new UserException( "Must have a src attribute present", PalooseException::SITEMAP_TRANSFORMER_PARSE_ERROR, $inDOM );
      }
   }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      parent::run( $inVariableStack, $inURL, $inQueryString, $inDOM );
      $documentDOM = $inDOM;
      
      $fullXSLFileName = StringResolver::expandString( $inVariableStack, $this->gSrc );
      $fullXSLFileName = StringResolver::expandPseudoProtocols( $fullXSLFileName );
      $validDOM = false;
      $validXSLFile = false;
      $validParameters = false;
      $requestParameterModule = Environment::$modules[ 'request-param' ];
      $isCachable = $this->gComponent->gIsCachable;
      $useReqParams = Utilities::normalizeBoolean( $this->gComponent->getUseRequestParametersFlag() );
      
      
      $localCachable = $requestParameterModule->get( CacheFile::CACHABLE_ATTRIBUTE_NAME );
      if ( $useReqParams && $localCachable != NULL ) {
         $localCachable = Utilities::normalizeBoolean( $localCachable );
         $isCachable = $localCachable;
      }
      $fullFileName = "";
      if ( $isCachable ) {
         $this->gCacheFile = new CacheFile( Environment::$configuration[ 'cacheDir' ] );
         
         $fullFileName = $documentDOM->documentElement->getAttribute( "__file" );
         $validDOM = ( $documentDOM->documentElement->getAttribute( "__status" ) == CacheFile::CACHE_VALID );
         
         $cacheKey = CacheFile::getHash( $fullFileName . $fullXSLFileName );
         
         
         $oldXSLTimeStamp = ( $isCachable ) ? $this->gCacheFile->getTimeStamp( $cacheKey ) : 0;
         $XSLTimeStamp = filemtime( $fullXSLFileName );
         $validXSLFile = ( $oldXSLTimeStamp == $XSLTimeStamp );
   
         
         $oldParametersHash = ( $isCachable ) ? $this->gCacheFile->getParameterHash( $cacheKey ) : 0;
         $newParameterHash = CacheFile::getHash( $this->gParameters->toString() );
         $validParameters = ( $oldParametersHash == $newParameterHash );
      }
      if ( $validDOM && $validXSLFile && $validParameters && $isCachable ) {
         $documentDOM = $this->gCacheFile->getCacheDOM( $cacheKey );
         $documentDOM->documentElement->setAttribute( "__status", CacheFile::CACHE_VALID );
      } else {
          
         $xslScript = new DomDocument(); 
         $xslProcessor = new XsltProcessor();
         $xslProcessor->registerPHPFunctions();
         try {
            if ( !file_exists( $fullXSLFileName ) ) {
                 throw new UserException(
                    "Cannot find XSL file '$fullXSLFileName'",
                    PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
            }
           
            $check = new XML_check();
            if ( !$check->check_url( $fullXSLFileName ) ) {
               throw new RunException(
                  "Invalid transform file '$fullXSLFileName'" .
                  "\n" . $check->get_full_error(),
                  PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
            }
            if ( !$xslScript->load( $fullXSLFileName ) ) {
               throw new RunException(
                  "Could not load transform file '$fullXSLFileName'" .
                  "\n" . $check->get_full_error(),
                  PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
            }
            $xslProcessor->importStylesheet( $xslScript );
            if ( !empty( $this->gParameters->getParameterList() ) ) {
               $parameterList = $this->gParameters->getParameterList();
               foreach ( $parameterList as $name => $value ) {
                  $expandedValue = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $value ) );
                  
                  $globalsModule = Environment::$modules[ 'global' ];
                  $globalsModule->add( $name, $value );
                  if ( $xslProcessor->setParameter( null, $name, $expandedValue ) === false ) {
                     throw new InternalException( "Problem setting parameter '$name' with value ' $value'",
                        PalooseException::SITEMAP_TRANSFORMER_RUN_ERROR );
                  }
               }
            }
  
            $documentDOM = $xslProcessor->transformToDoc( $documentDOM );
            if ( $documentDOM->documentElement ) {
               $documentDOM->documentElement->setAttribute( "__status", CacheFile::CACHE_INVALID );
            }
            
            if ( $isCachable ) {
               $this->gCacheFile->updateDOM2Cache( $cacheKey, $XSLTimeStamp, $documentDOM, $newParameterHash );
            }
         } catch ( RunException $e ) {
              throw new RunException( $e->getMessage(), $e->getCode() );
         }
      }
      if ( $documentDOM->documentElement ) {
         $documentDOM->documentElement->setAttribute( "__file", $fullFileName );
      }
      
      return $documentDOM;
   }
   
   
   
  
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
}
 
class _TRAXTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
      $this->gPackageName = "TRAXTransformer";
   }
}
?>
