<?php
 
require_once( PALOOSE_LIB_DIRECTORY . "/environment/Environment.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/pipelines/PipeElementInterface.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/sitemap/Parameter.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/transforming/TransformerPipeElement.php" );
require_once( PALOOSE_LIB_DIRECTORY . "/external/class_xml_check.php" );
 
class VariableTransformer extends TransformerPipeElement implements PipeElementInterface
{
       
   
   
   
   public function __construct( DOMDocument $inDOM, $inType = "", $inSrc = "", $inComponent = NULL )
   {
        try {
         parent::__construct( $inDOM, $inType, $inSrc, $inComponent );
        } catch ( UserException $e ) {
           throw new UserException( $e->getMessage(), $e->getCode(), $e->getDOMScrap() );
        }
       
    }
   
   
   
   public function run( VariableStack $inVariableStack, $inURL, $inQueryString, DOMDocument $inDOM )
   {
      
      $documentDOM = $inDOM;
      $rootNode = $documentDOM->documentElement;
      $this->gTranslatedDOM = new DOMDocument();
      $elementNS = $rootNode->namespaceURI;
      $newElement = $this->gTranslatedDOM->createElementNS( $elementNS, $rootNode->nodeName );
      $this->gTranslatedDOM->appendChild( $newElement );
      try {
         $this->processNode( $inVariableStack, $rootNode, $newElement );
         
         return $this->gTranslatedDOM;
      } catch ( RunException $e ) {
         throw new RunException( $e->getMessage(), $e->getCode(), NULL );
      }
   }
   
   
   
   
   private function processNode( VariableStack $inVariableStack, DOMNode $inParentNode, DOMNode &$inNewParentNode )
   {
     
      if ( $inParentNode->nodeType == XML_ELEMENT_NODE ) {
         
         if ( $inParentNode->hasAttributes() ) {
            foreach ( $inParentNode->attributes as $attribute ) {
               $attribName = StringResolver::expandString( $inVariableStack, $attribute->name );
               $attribValue = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $attribute->value ) );
               $inNewParentNode->setAttribute( $attribName, $attribValue );
            }
         }
      }
      
      
      $children = $inParentNode->childNodes;
      foreach ( $children as $child ) {
         if ( $child->nodeType == XML_ELEMENT_NODE ) {
           
           $newElementNode = $this->gTranslatedDOM->createElementNS( $child->namespaceURI, $child->nodeName );
           
           $this->processNode( $inVariableStack, $child, $newElementNode );
           $inNewParentNode->appendChild( $newElementNode );
         } else if ( $child->nodeType == XML_TEXT_NODE ) {
            
            
            $expandedContent = StringResolver::expandPseudoProtocols( StringResolver::expandString( $inVariableStack, $child->textContent ) );
            $newTextNode = $this->gTranslatedDOM->createTextNode( $expandedContent );
            $inNewParentNode->appendChild( $newTextNode );
         }
      }
   }
  
   
   
   
  
   public function toString()
   {
      return parent::toStringWithType( $this->gGenericType );
   }
}
 
class _VariableTransformer extends Transformer {
    
   
   
   
   
   public function __construct( $inName, $inSrc, DOMNode $inParameterNode, $inIsCachable )
   {
      parent::__construct( $inName, $inSrc, $inParameterNode, $inIsCachable );
       $this->gGenericType = PipeElement::TRANSFORMER;
        $this->gPackageName = "VariableTransformer";
      
   }
}
?>
