<?php
/**
 * @package paloose
 */
/*
Plugin Name: paloose
Plugin URI: http://www.paloose.org/
Description: Paloose plugin provides a Cocoon like extension to WordPress to handle XML files using XSL.
Version: 1.0.1b7
Author: Hugh Field-Richards
Author URI: http://automattic.com/wordpress-plugins/
License: GPLv2 (only the plugin, Paloose is issued under GPLV3)
*/
/*
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 

require_once( dirname( __FILE__ ) . '/paloose-admin/options.php');
require_once( dirname( __FILE__ ) . '/paloose-includes/functions.php');

/** The instance of the main PaloosePlugin class */
$gPaloosePlugin = null;

/** The instance of the main PaloosePlugin admin class */
$gPaloosePluginAdmin = null;

$gAdminPageHook = "";

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 * The main Paloose Plugin class where everything happens.
 * 
 * @package paloose
 * @subpackage PaloosePlugin
 */
 
if ( !class_exists( 'PaloosePlugin' ) ) {

   class PaloosePlugin
   {

      /** The instance of the admin class */
      var $paloosePluginAdmin = "PaloosePluginAdminOptions";
   
     //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
     //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
     /**
      * The constructor for the Paloose class.
      */
    
      function PaloosePlugin( $inPaloosePluginAdmin ) {
         // Store admin class locally in this class
         $this->paloosePluginAdmin = $inPaloosePluginAdmin;
      } // PaloosePlugin constructor

     //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
     //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
     /**
      * Initialise the plugin.
      */
    
      function init() {
         $this->paloosePluginAdmin->getAdminOptions();
      } // init

   } // PaloosePlugin

}

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 * Set up the admin class.
 */
 
if ( class_exists( "PaloosePluginAdmin") ) {
   $gPaloosePluginAdmin = new PaloosePluginAdmin();
} // fi

/**
 * Activate the plugin.
 */
 
if ( class_exists( "PaloosePlugin") ) {
   $gPaloosePlugin = new PaloosePlugin( $gPaloosePluginAdmin );
} // fi

/**
 * Add the admin panel
 */
 
if ( !function_exists( "addPalooseAdminPluginPanel" ) ) {

   function addPalooseAdminPluginPanel() {
		global $gPaloosePlugin;
		global $gPaloosePluginAdmin;
		global $gAdminPageHook;

      // Check that class does exist
		if ( !isset( $gPaloosePlugin ) ) {
			return;
		} // fi
		// Check whether current user is authorised for this page
	   if ( !current_user_can( 'manage_options' ) ) {
         wp_die( __( 'Sorry, you do not have sufficient permissions to access this page.' ) );
      } // fi
		// Make sure add options stuff is present. Probably overkill as it should be part of WordPress
		if ( function_exists( 'add_options_page' ) ) {
	      $gAdminPageHook = add_options_page( 
            'Paloose Plugin Options',                        // page_title
            'Paloose Plugin',                                // menu_title
            9,                                               // access_level/capability
            basename( __FILE__ ),                            // unique id (file name)
            array( &$gPaloosePluginAdmin, 'printAdminPage' ) // reference to the instance variable $gPaloosePluginAdmin and call the init() function
	      );
	      add_action( "admin_print_scripts-$gAdminPageHook", array( &$gPaloosePluginAdmin, 'addScriptsStyles' ) );
		} // fi
		
	} // initPalooseAdminPluginPanel

} // fi

//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
//-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
/**
 * Activate the actions etc..
 */
 
if ( isset( $gPaloosePlugin ) ) {
   global $gPaloosePluginAdmin;

   // Add all the actions
   add_action( 'activate_paloose/paloose-xml-processor.php', array( &$gPaloosePlugin, 'init' ) );
   // addPalooseAdminPluginPanel should be loaded here so add it to the admin menu
   add_action( 'admin_menu', 'addPalooseAdminPluginPanel' );
}

?>