<?php

/*
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*

   Date                   Who    Changes
   ==========================================================================

   15th March 2007        HSFR   Created
   
   LICENSE:
   
   This software is free software; you can redistribute it
   and/or modify it under the terms of the GNU Lesser General
   Public License as published by the Free Software Foundation;
   either version 2.1 of the License, or (at your option) any
   later version.
   
   This software is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with this software; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

*/

require_once( PALOOSE_LIB_DIRECTORY . "/flows/Continuations.php" );

// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
// -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
/**
 *
 */
 
class EditUser extends Continuations {

   /** Logger instance for this class */
   private $gLogger;

   /** The data model for the form. Rather than follow the structure of the XML data
    this is a "flat" structure. The XML is it associated with in this case is:
      
      <user>
         <username>hsfr</username>
         <password>xxxxxx</password>
         <data>
            <roles>editPage,displayUsers</roles>
            <fullname>Hugh Field-Richards</fullname>
            <email>hsfr@hsfr.org.uk</email>
            <address>
               <line-1>x xxx xxxx xxxxx</line-1>
               <line-2>xxxxxxxxxx</line-2>
               <postcode>WRxx xxx</postcode>
            </address>
         </data>
      </user>

      */

   private $gAddFormModel = array (
      "username" => "",
      "password" => "",
      "passwordCheck" => "",  // Extra field not appearing above
      "fullname" => "",
      "email" => "",
      "line-1" => "",
      "line-2" => "",
      "postcode" => "",
      "comments" => "",
      "roles" => ""
   );

   // The following is just one way of handling errors. You could have a system of
   // conditional statements in-line with the code and test each case explicitly.
   // The choice is up to you. I have elected to have a keyed array of error messages
   private $dataRequired = array (
      "username" => "Must enter a username",
      "password" => "Must enter a password",
      "passwordCheck" => "Must enter a password check",
   );

   // -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   // -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   /**
    * Make a new instance of the continuation.
    * 
    * @throws RunException if there is no current session in progress.
    */
    
   function __construct()
   {
 		$this->gLogger = Logger::getLogger( __CLASS__ );
    	parent::__construct( $this->gAddFormModel );
   }

   // -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   // -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   /**
    */
    
   function edit()
   {
     	global $gModules;

 		$this->gLogger->debug( "Editing user, continuation: " . $this->gContinuation );
      $requestParameterModule = $gModules[ 'request-param' ];
      $this->gViolations = array();

      $finished = false;
      while ( !$finished ) {
         $errors = false;
    		switch ( $this->gContinuation ) {
    		
    		   case 0 : // Sets up the data on the fiirst screnn (only called once to set up session/flow variables)
    		      $this->sendPage(
    		         "editUser-0.html", 
    		         $this->gAddFormModel, 
    		         $this->gContinuation + 2,    // Skip a stage as we want the want to use the addUser method
    		         $this->gViolations ); 
    		      $finished = true;
    		      break;
   
    		   case 1 : // Invoked if the data on the second screen (case 2) is wrong
    		      $this->sendPage(
    		         "editUser-1.html",        
    		         $this->gAddFormModel,
    		         ++$this->gContinuation,
    		         $this->gViolations );
    		      $finished = true;
    		      break;
   
    		   case 2 : 
       		   // Error processing here
          		// First I have to check that there are contents in the required fields. I could leave this
          		// out the "isset" but some PHP systems will have strict error checking and produce extraneous
          		// error messages on the page.
          		
          		foreach ( $this->dataRequired as $key => $msg ) {
          		   if ( !isset( $this->gAddFormModel[ $key ] ) or strlen( $this->gAddFormModel[ $key ] ) == 0 ) {
                     $this->gViolations[ $key ] = $msg;
             	      $errors = true;
             		}
               }

          	   // Now check for remaining errors. Adjust the order of the checks to suit local conditions.
          	   // The last check is what is displayed.
               if ( $errors === false ) {
                  if ( strlen( $this->gAddFormModel[ 'password' ] ) < 8 ) {
                     $this->gViolations[ 'password' ] = "Password must be more than 8 characters long";
          		      $errors = true;
                  }
                  if ( $this->gAddFormModel[ 'password' ] != $this->gAddFormModel[ 'passwordCheck' ] ) {
                     $this->gViolations[ 'password' ] = "Password does not check";
          		      $errors = true;
                  }
               }
               if ( $errors ) {
                  $this->gContinuation--;
       		      $finished = false;
               } else {
                  // No errors so next page so send next page
                  $this->sendPage(
                     "editUser-2.html",
                     $this->gAddFormModel,
                     ++$this->gContinuation,
                     $this->gViolations );
    	   	      $finished = true;
    	   	   }
    		      break;
   
    		   case 3 : 
               // No errors to record from optional data so send next page
               $this->sendPage(
                  "editUser-3.html",
                  $this->gAddFormModel,
                  ++$this->gContinuation,
                  $this->gViolations );
       	      $finished = true;
    		      break;
   
    		   case 4 : 
               // Data confirmed so write into admin user's file etc
               $this->sendPage(
                  "editUser-4.html",
                  $this->gAddFormModel,
                  ++$this->gContinuation,
                  $this->gViolations );
       	      $finished = true;
    		      break;
   
         }
      }
   }

   //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
   //-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* 
   /**
    * Get the parameters as an array.
    *
	 * @return string value of variable or NULL is not exist.
    */
    
	public function toString()
	{
	   $mess = "";
	   foreach ( $this->gAddFormModel as $name => $value ) {
	      $mess .= $name . " : " . $value . "\n";
	   }
	   return $mess;
	}	

}
?>