<?xml version="1.0" encoding="ISO-8859-1" ?>

<!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
<!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
<!--
    
    DSD-2 to HTML Pretty Printer
    
    This XSLT style sheet pretty prints a DSD-2 schema. It has been tested
    on the current Xalan processor and renders to most browsers (Safari, IE, 
    Mozilla, Netscape etc). It was inspire by the DSD-1 pretty printer written
    by Nils Klarlund for the original DSD-1. Note that the DSD documentation
    that is processed is not in HTML but a personal XML format - this could
    be changed if anyone cares to. The actual format is relatively easy to
    understand if you look at this file and some of my DSD schemas.
    
    Author:
    	Name  : Hugh Field-Richards.
    	Email : hsfr@hsfr.org.uk
    
    LICENSE:
    
    This software is free software; you can redistribute it
    and/or modify it under the terms of the GNU Lesser General
    Public License as published by the Free Software Foundation;
    either version 2.1 of the License, or (at your option) any
    later version.
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with this software; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA

-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:dsd="http://www.brics.dk/DSD/2.0"
    xmlns:xi="http://www.w3.org/2001/XInclude" xmlns:m="http://www.brics.dk/DSD/2.0/meta"
    xmlns:doc="http://www.paloose.org/schemas/DSD/DSDs/doc" xmlns:xhtml="http://www.w3.org/1999/xhtml"
    xmlns:a="http://relaxng.org/ns/annotation/1.0" xmlns:dc="http://purl.org/dc/elements/1.1/"
    xmlns:map="http://apache.org/cocoon/sitemap/1.0" version="1.0">

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*- BOOLEAN EXPRESSIONS -*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!-- 
        A boolean expression describes elements in the instance document:

        BOOLEXP ::= <and> BOOLEXP* </and>
                  | <or> BOOLEXP* </or>
                  | <not> BOOLEXP </not>
                  | <imply> BOOLEXP BOOLEXP </imply>
                  | <equiv> BOOLEXP* </equiv> (XOR)
                  | <one> BOOLEXP* </one> (XNOR)
                  | <parent> BOOLEXP </parent>
                  | <ancestor> BOOLEXP </ancestor>
                  | <child> BOOLEXP </child>
                  | <descendant> BOOLEXP </descendant>
                  | <this/>
                  | <element ( name="PENAME" )? />
                  | <attribute ( name="PANAME" )? > REGEXP? </attribute>
                  | <contents> REGEXP* </contents>
                  | <boolexp ref="PENAME"/>

         Additional syntactic restrictions:

                this must have a unique or pointer ancestor; and
                every attribute expression that contains a REGEXP also contains a name property. 
    
        This is a recursive tempalte for processing the above.
        
        param inIsConditionBlock true if this is the condition block at thhe start of a rule (thus needs 
                                 highlighting and raw/pretty button inserted). When called as part of the
                                 recursion this defaults to false.
    -->

    
    <xsl:template match="
        *[local-name() = 'and'
        or local-name() = 'or'
        or local-name() = 'not'
        or local-name() = 'imply'
        or local-name() = 'equiv'
        or local-name() = 'one'
        or local-name() = 'parent'
        or local-name() = 'ancestor'
        or local-name() = 'child'
        or local-name() = 'descendant'
        or local-name() = 'this'
        or local-name() = 'root'
        or local-name() = 'element'
        or local-name() = 'attribute'
        or local-name() = 'contents'
        or ( local-name() = 'boolexp' and @ref )]">
        <xsl:call-template name="boolexp-definitions"/>
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!--
     -->
    
    <xsl:template name="boolexp-definitions">
        <xsl:param name="inIsConditionBlock" select="false()"/>
        
        <xsl:variable name="theNode" select="local-name(child::*[1])"/>
        
        <xsl:element name="xhtml:div">
            <xsl:attribute name="class">indentedBlock</xsl:attribute>
            
            <xsl:choose>
                <xsl:when test="$theNode = 'and' or $theNode = 'or' or $theNode = 'equiv' or $theNode = 'one'">
                    <xsl:call-template name="multi-boolexp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when
                    test="$theNode = 'parent' or $theNode = 'ancestor' or $theNode = 'child' or $theNode = 'descendant' or $theNode = 'not'">
                    <xsl:call-template name="unary-boolexp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'imply'">
                    <xsl:call-template name="imply-boolexp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'contents'">
                    <xsl:call-template name="contents-boolexp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'attribute'">
                    <xsl:call-template name="attribute-boolexp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'element'">
                    <xsl:call-template name="element-exp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'root'">
                    <xsl:call-template name="root-exp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'this'">
                    <xsl:call-template name="this-exp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="$theNode = 'boolexp'">
                    <xsl:call-template name="boolexp-exp">
                        <xsl:with-param name="inIsConditionBlock" select="$inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:otherwise>
                    <xsl:call-template name="render-element-name"/>
                    <xsl:call-template name="render-id-attrib"/>
                </xsl:otherwise>
            </xsl:choose>
        </xsl:element>
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!--
     -->
    
    <xsl:template name="boolexp">
        <xsl:param name="inIsConditionBlock" select="false()"/>
    
        <xsl:variable name="inElementName" select="local-name(.)"/>
      
        <xsl:element name="div">
            <xsl:attribute name="class">indentedBlock</xsl:attribute>
            
            <xsl:choose>
                <xsl:when
                    test="local-name(.) = 'and' or local-name(.) = 'or' or local-name(.) = 'equiv' or local-name(.) = 'one'">
                    <xsl:call-template name="multi-boolexp"/>
                </xsl:when>
    
                <xsl:when
                    test="local-name(.) = 'parent' or local-name(.) = 'ancestor' or local-name(.) = 'child' or local-name(.) = 'descendant' or local-name(.) = 'not'">
                    <xsl:call-template name="unary-boolexp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'contents'">
                    <xsl:call-template name="contents-boolexp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'imply'">
                    <xsl:call-template name="contents-boolexp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'attribute'">
                    <xsl:call-template name="attribute-boolexp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'element'">
                    <xsl:call-template name="element-exp"/>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'root'">
                    <xsl:call-template name="root-exp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'this'">
                    <xsl:call-template name="this-exp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:when test="local-name(.) = 'boolexp'">
                    <xsl:call-template name="boolexp-exp">
                        <xsl:with-param name="inIsConditionBlock"/>
                    </xsl:call-template>
                </xsl:when>
                
                <xsl:otherwise/>
            </xsl:choose>      
        </xsl:element>
    </xsl:template>

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    
    <xsl:template name="unary-boolexp">
        <xsl:param name="isCondition" select="false()"/>

        <xsl:call-template name="render-declare-name"/>
        <xsl:call-template name="render-id-attrib"/>
        
        <xsl:for-each select="*[not(position() > 1)]">
            <xsl:call-template name="boolexp">
                <xsl:with-param name="isCondition" select="$isCondition"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    <!-- 
       Binary operation (or more) 
    -->

    <xsl:template name="multi-boolexp">
        <xsl:param name="inIsConditionBlock" select="false()"/>

        <xsl:variable name="inElementName" select="local-name(.)"/>

        <xsl:call-template name="outputElement">
            <xsl:with-param name="inElementName" select="$inElementName"/>
        </xsl:call-template>

        <xsl:for-each select="*">
            <xsl:call-template name="boolexp">
                <xsl:with-param name="isCondition"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    
    <xsl:template name="element-exp">
        <xsl:param name="inIsConditionBlock" select="false()"/>
        
        <xsl:choose>
            <xsl:when test="@name">
                <xsl:variable name="elementName">
                    <xsl:if test="@prefix">
                        <xsl:value-of select="@prefix"/>
                    </xsl:if>
                    <xsl:value-of select="@name"/>
                </xsl:variable>
                
                <xsl:call-template name="outputDefinitionAndValue">
                    <xsl:with-param name="inElementName" select="'element'"/>
                    <xsl:with-param name="inValue" select="$elementName"/>
                </xsl:call-template>
            </xsl:when>
            
            <xsl:otherwise>
                <span class="chars">any element</span>
                <xsl:call-template name="render-namespace-attrib"/>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->

    <xsl:template name="attribute-boolexp">
        <xsl:param name="inIsConditionBlock" select="false()"/>
      
        <xsl:call-template name="outputDefinitionAndValue">
            <xsl:with-param name="inElementName" select="local-name(.)"/>
            <xsl:with-param name="inValue" select="@name"/>
        </xsl:call-template>

         <xsl:for-each select="*">
            <xsl:call-template name="regexp"/>
        </xsl:for-each>
    </xsl:template>

     <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->

    <xsl:template name="imply-boolexp">
        <xsl:param name="isCondition"/>
        <xsl:call-template name="render-element-name"/>
        <xsl:call-template name="render-id-attrib"/>
        <xsl:for-each select="*[not(position() > 2)]">
            <xsl:call-template name="boolexp">
                <xsl:with-param name="isCondition" select="$isCondition"/>
            </xsl:call-template>
        </xsl:for-each>
    </xsl:template>

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    
   <xsl:template name="this-exp">
        <xsl:param name="isCondition"/>
        <xhtml:span class="chars">this element</xhtml:span>
    </xsl:template>

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    
   <xsl:template name="root-exp">
        <xsl:param name="isCondition"/>
        <xhtml:span class="chars">root element</xhtml:span>
    </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    
    <xsl:template name="boolexp-exp">
        <xsl:param name="isCondition" select="false()"/>
        
        <xsl:variable name="elementName" select="local-name(.)"/>
        <xsl:variable name="refName" select="@ref"/>
        
        <xsl:call-template name="outputRuleWithRef">
            <xsl:with-param name="inElementName" select="$elementName"/>
            <xsl:with-param name="inRefName" select="$refName"/>
        </xsl:call-template>
        
     </xsl:template>

    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- -->
    
    <xsl:template name="contents-boolexp">
        <xsl:param name="isCondition"/>
        <xsl:call-template name="render-element-name"/>
        <xsl:call-template name="render-id-attrib"/>
        <xsl:for-each select="*">
            <xsl:call-template name="regexp"/>
        </xsl:for-each>
    </xsl:template>


</xsl:stylesheet>
