<?xml version="1.0"?>

<!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
<!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
<!--
   
   Builds the header information for the pages.
   
   Author:
      Name  : Hugh Field-Richards
      Email : hsfr@hsfr.org.uk
   
  -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*- Copyright -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*
   
   This software and the contents of the site is Copyright Hugh Field-Richards 2008.
    
-->

<xsl:stylesheet
    xmlns:xsl="http://www.w3.org/1999/XSL/Transform" 
    xmlns:graphic="http://www.hsfr.org.uk/Schema/Graphic"
    xmlns:list="http://www.hsfr.org.uk/Schema/List"
    xmlns:link="http://www.hsfr.org.uk/Schema/Link"
    xmlns:text="http://www.hsfr.org.uk/Schema/Text" 
    xmlns:news="http://www.hsfr.org.uk/Schema/News"
    xmlns:page="http://www.hsfr.org.uk/Schema/Page" 
    xmlns:paloose="http://www.paloose.org/schemas/Paloose/1.0"
    version="1.0">
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-* BUILD HTML HEAD *-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- 
        @param inTitle             the title of the page
        @param inStylesList        the list of style files to output (file-1,file-2,file-3, ...)
        @param inStylesDir         the folder where the style files are stored
        @param inScriptsList       the list of script files to output (file-1,file-2,file-3, ...)
        @param inScriptsDir        the folder where the script files are stored
        @param inJQueryScriptsList the list of JQuery script files to output (file-1,file-2,file-3, ...)
        @param inJQueryScriptsDir  the folder where the JQuery script files are stored
    -->
    
    <xsl:template name="buildHeader.htmlHead">
        <xsl:param name="inTitle"/>
        <xsl:param name="inStylesList"/>
        <xsl:param name="inStylesDir"/>
        <xsl:param name="inScriptsList"/>
        <xsl:param name="inScriptsDir"/>
        <xsl:param name="inJQueryScriptsList"/>
        <xsl:param name="inJQueryScriptsDir"/>
        
        <xsl:element name="meta">
            <xsl:attribute name="name">DC.creator</xsl:attribute>
            <xsl:attribute name="content">Hugh Field-Richards</xsl:attribute>
        </xsl:element>
        
        <xsl:element name="meta">
            <xsl:attribute name="name">DC.language</xsl:attribute>
            <xsl:attribute name="scheme">RFC1766</xsl:attribute>
            <xsl:attribute name="content">en</xsl:attribute>
        </xsl:element>
        
        <xsl:element name="meta">
            <xsl:attribute name="name">DC.Title</xsl:attribute>
            <xsl:attribute name="lang">en</xsl:attribute>
            <xsl:attribute name="content">
                <xsl:value-of select="$inTitle"/>
            </xsl:attribute>
        </xsl:element>
        
        <xsl:element name="title">
            <xsl:value-of select="$inTitle"/>
        </xsl:element>
        
        <xsl:element name="link">
            <xsl:attribute name="rel">icon</xsl:attribute>
            <xsl:attribute name="type">image/png</xsl:attribute>
            <xsl:attribute name="href">paloose-favicon.png</xsl:attribute>
        </xsl:element>
        
        <xsl:call-template name="buildHeader.outputScriptsList">
            <xsl:with-param name="inScriptsList" select="$inJQueryScriptsList"/>
            <xsl:with-param name="inScriptsDir" select="$inJQueryScriptsDir"/>
        </xsl:call-template>
        
        <xsl:call-template name="buildHeader.outputScriptsList">
            <xsl:with-param name="inScriptsList" select="$inScriptsList"/>
            <xsl:with-param name="inScriptsDir" select="$inScriptsDir"/>
        </xsl:call-template>
        
        <xsl:call-template name="buildHeader.outputStylesList">
            <xsl:with-param name="inStylesList" select="$inStylesList"/>
            <xsl:with-param name="inStylesDir" select="$inStylesDir"/>
        </xsl:call-template>
        
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- 
      Output a list of comma separated style files as
      
      <link href="file-1" ... />
      <link href="file-2" ... />
      <link href="file-3" ... />
      
       param inStylesList  the list of style files to output (file-1,file-2,file-3, ...)
       param inStylesDir   the folder where the style files are stored
    -->
    
    <xsl:template name="buildHeader.outputStylesList">
        <xsl:param name="inStylesList"/>
        <xsl:param name="inStylesDir"/>
        
        <xsl:choose>
            <xsl:when test="contains($inStylesList, ',')">
                <xsl:variable name="firstStyle" select="substring-before($inStylesList, ',')"/>
                <xsl:variable name="remainderStyles" select="substring-after($inStylesList, ',')"/>
                <xsl:call-template name="buildHeader.outputStyle">
                    <xsl:with-param name="inStyle" select="$firstStyle"/>
                    <xsl:with-param name="inStylesDir" select="$inStylesDir"/>
                </xsl:call-template>
                <xsl:call-template name="buildHeader.outputStylesList">
                    <xsl:with-param name="inStylesList" select="$remainderStyles"/>
                    <xsl:with-param name="inStylesDir" select="$inStylesDir"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="buildHeader.outputStyle">
                    <xsl:with-param name="inStyle" select="$inStylesList"/>
                    <xsl:with-param name="inStylesDir" select="$inStylesDir"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- 
      Output a single style files as <link href="inStyle" ... />
      
      param inStyle the styles file name
   -->
    
    <xsl:template name="buildHeader.outputStyle">
        <xsl:param name="inStyle"/>
        <xsl:param name="inStylesDir"/>
        <xsl:element name="link">
            <xsl:attribute name="href">
                <xsl:value-of select="concat($inStylesDir, $inStyle)"/>
            </xsl:attribute>
            <xsl:attribute name="rel">stylesheet</xsl:attribute>
            <xsl:attribute name="type">text/css</xsl:attribute>
        </xsl:element>
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- 
      Output a list of comma separated stscriptyle files as
      
      <link href="file-1" ... />
      <link href="file-2" ... />
      <link href="file-3" ... />
      
       param inScriptsList  the list of script files to output (file-1,file-2,file-3, ...)
       param inScriptsDir   the folder where the script files are stored
    -->
    
    <xsl:template name="buildHeader.outputScriptsList">
        <xsl:param name="inScriptsList"/>
        <xsl:param name="inScriptsDir"/>
        
        <xsl:choose>
            <xsl:when test="contains($inScriptsList, ';')">
                <xsl:variable name="firstScript" select="substring-before($inScriptsList, ';')"/>
                <xsl:variable name="remainderScripts" select="substring-after($inScriptsList, ';')"/>
                <xsl:call-template name="buildHeader.outputScript">
                    <xsl:with-param name="inScript" select="$firstScript"/>
                    <xsl:with-param name="inScriptsDir" select="$inScriptsDir"/>
                </xsl:call-template>
                <xsl:call-template name="buildHeader.outputScriptsList">
                    <xsl:with-param name="inScriptsList" select="$remainderScripts"/>
                    <xsl:with-param name="inScriptsDir" select="$inScriptsDir"/>
                </xsl:call-template>
            </xsl:when>
            <xsl:otherwise>
                <xsl:call-template name="buildHeader.outputScript">
                    <xsl:with-param name="inScript" select="$inScriptsList"/>
                    <xsl:with-param name="inScriptsDir" select="$inScriptsDir"/>
                </xsl:call-template>
            </xsl:otherwise>
        </xsl:choose>
    </xsl:template>
    
    <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
    <!-- 
      Output a single script files as <script type="text/javascript" src="js/prototype.js" ... />
      
       param inScript       the script file name
       param inScriptsDir   the folder where the script files are stored
   -->
    
    <xsl:template name="buildHeader.outputScript">
        <xsl:param name="inScript"/>
        <xsl:param name="inScriptsDir"/>
        
        <xsl:element name="script">
            <xsl:attribute name="src">
                <xsl:value-of select="concat($inScriptsDir, $inScript)"/>
            </xsl:attribute>
            <xsl:attribute name="type">text/javascript</xsl:attribute>
        </xsl:element>
    </xsl:template>
    
</xsl:stylesheet>
