<?xml version="1.0" encoding="UTF-8"?>

<!--
  Copyright 1999-2004 The Apache Software Foundation

  Licensed under the Apache License, Version 2.0 (the "License");
  you may not use this file except in compliance with the License.
  You may obtain a copy of the License at

      http://www.apache.org/licenses/LICENSE-2.0

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.
-->

<!--+
    | XSLT REC Compliant Version of IE5 Default Stylesheet
    |
    | Original version by Jonathan Marsh (jmarsh@microsoft.com)
    | Conversion to XSLT 1.0 REC Syntax by Steve Muench (smuench@oracle.com)
    | Added script support by Andrew Timberlake (andrew@timberlake.co.za)
    | Cleaned up and ported to standard DOM by Stefano Mazzocchi (stefano@apache.org)
    | More cleansing by Hugh Field-Richards (hsfr@hsfr.org.uk)
    | Complete rewrite by Hugh Field-Richards (hsfr@hsfr.org.uk)
    |
    | CVS $Id: xml2html.xslt,v 1.10 2004/05/01 13:20:55 joerg Exp $
    +-->
    
<!DOCTYPE xsl:stylesheet [
   <!ENTITY lt			"&#38;#60;" >		<!-- less-than sign -->
   <!ENTITY gt			"&#62;" >			<!-- greater-than sign -->
	<!ENTITY ndash		"&#x2013;" >		<!-- en dash -->
	<!ENTITY mdash		"&#x2014;" >		<!-- em dash -->
	<!ENTITY ldquo  	"&#x201c;">			<!-- Left Double Quote -->
   <!ENTITY rdquo  	"&#x201d;">			<!-- Right Double Quote -->
   <!ENTITY lsquo  	"&#x2018;">			<!-- Left Single Quote -->
   <!ENTITY rsquo  	"&#x2019;">			<!-- Right Single Quote -->
   <!ENTITY nbsp		"&#160;" >			<!-- no break (required) space -->
   <!ENTITY apos		"&#39;" >			<!-- apostrophe -->
	<!ENTITY agrave	"&#224;" >			<!-- small a, grave accent -->   
	<!ENTITY aacute	"&#225;" >			<!-- small a, acute accent -->
	<!ENTITY acirc		"&#226;" >			<!-- small a, circumflex accent -->
   <!ENTITY auml		"&#228;" >			<!-- small a, dieresis or umlaut mark -->
	<!ENTITY ouml    	"&#246;" >			<!-- small o, dieresis or umlaut mark -->
   <!ENTITY egrave	"&#232;" >			<!-- small e, grave accent -->
   <!ENTITY eacute	"&#233;" >			<!-- small e, acute accent -->
   <!ENTITY ecirc		"&#234;" >			<!-- small e, circumflex accent -->
   <!ENTITY euml		"&#235;" >			<!-- small e, dieresis or umlaut mark -->
   <!ENTITY frac14	"&#188;" >			<!-- fraction one-quarter -->
   <!ENTITY rcaron   "&#x159;" >       <!-- small r, caron -->
   <!ENTITY ntilde   "&#241;" >        <!-- small n, tilde -->
   <!ENTITY frac12   "&#189;" >        <!-- fraction one-half -->
   <!ENTITY szlig    "&#223;" >        <!-- small sharp s, German (sz ligature) -->
   <!ENTITY utrif    "&#x25B4;" >      <!--/blacktriangle =up tri, filled-->
   <!ENTITY dtrif    "&#x25BE;" >      <!--/blacktriangledown =dn tri, filled-->
   <!ENTITY ltrif    "&#x25C2;" >      <!--/blacktriangleleft R: =l tri, filled-->
   <!ENTITY rtrif    "&#x25B8;" >      <!--/blacktriangleright R: =r tri, filled-->
] >

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns:xd="http://www.oxygenxml.com/ns/doc/xsl" exclude-result-prefixes="xd"
   version="1.0">

   <xd:doc scope="stylesheet">
      <xd:desc>
         <xd:p><xd:b>Created on:</xd:b> Sep 30, 2010</xd:p>
         <xd:p><xd:b>Author:</xd:b> Hugh Field-Richards</xd:p>
         <xd:p/>
      </xd:desc>
   </xd:doc>

   <xsl:output encoding="UTF-8" indent="yes"/>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Add your global variables here as required. The default values are set here as well.
   -->

   <xsl:variable name="gPageTitle" select="'Raw XML'"/>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Comma separated list of style files. Semicolon for style files - don't ask.
   -->

   <xsl:variable name="gCSSFilesDir" select="'/{ant:siteName}/resources/styles/'"/>
   <xsl:variable name="gCSSFiles" select="'prettyXML.css'"/>

   <xsl:variable name="gScriptFilesDir" select="'/{ant:siteName}/resources/scripts/'"/>
   <xsl:variable name="gScriptFiles" select="'jquery.js;prettyXML.js'"/>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      The root where it all starts. It should not be necessary to change anything in here.
      The main user change area is all within the buildBody etc.
   -->

   <xsl:template match="/">
      <xsl:element name="html">
         <xsl:attribute name="lang">en</xsl:attribute>
         <!-- Put html attributes here - for example language -->
         <xsl:element name="head">
            <xsl:call-template name="buildHeader.htmlHead">
               <xsl:with-param name="inTitle" select="$gPageTitle"/>
               <xsl:with-param name="inStylesList" select="$gCSSFiles"/>
               <xsl:with-param name="inStylesDir" select="$gCSSFilesDir"/>
               <xsl:with-param name="inScriptsList" select="$gScriptFiles"/>
               <xsl:with-param name="inScriptsDir" select="$gScriptFilesDir"/>
            </xsl:call-template>
         </xsl:element>
         <xsl:element name="body">
            <xsl:element name="a">
               <xsl:attribute name="name">topOfPage</xsl:attribute>
            </xsl:element>
            <xsl:element name="div">
               <xsl:attribute name="id">bodyFrame</xsl:attribute>
               <xsl:apply-templates/>
            </xsl:element>
         </xsl:element>
      </xsl:element>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
	   Any external XSL files to be included go here. These go at the end to make sure
	   the above override them.
   -->

   <xsl:include href="buildHeader.xsl"/>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match processing instructions
   -->

   <xsl:template match="processing-instruction()">
      <xsl:element name="div">
         <xsl:attribute name="class">e</xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&lt;?</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">processingInstruction</xsl:attribute>
            <xsl:value-of select="name(.)"/>
            <xsl:text> </xsl:text>
            <xsl:value-of select="."/>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>?&gt;</xsl:text>
         </xsl:element>
      </xsl:element>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match text
   <xsl:template match="text()">
      <xsl:element name="div">
         <xsl:attribute name="class">e</xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">textContent</xsl:attribute>
            <xsl:value-of select="."/>
         </xsl:element>
      </xsl:element>
   </xsl:template>
   -->

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match comments
   -->

   <xsl:template match="comment()">
      <xsl:element name="div">
         <xsl:attribute name="class">e</xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&lt;!--</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">c</xsl:attribute>
            <xsl:element name="pre">
               <xsl:value-of select="."/>
            </xsl:element>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>--></xsl:text>
         </xsl:element>
      </xsl:element>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match attributes
   -->

   <xsl:template match="@*">
      <xsl:element name="span">
         <xsl:attribute name="class">attributeName</xsl:attribute>
         <xsl:value-of select="name(.)"/>
      </xsl:element>
      <xsl:element name="span">
         <xsl:attribute name="class">attributeDelimiter</xsl:attribute>
         <xsl:text>="</xsl:text>
      </xsl:element>
      <xsl:element name="span">
         <xsl:attribute name="class">attributeValue</xsl:attribute>
         <xsl:value-of select="."/>
      </xsl:element>
      <xsl:element name="span">
         <xsl:attribute name="class">attributeDelimiter</xsl:attribute>
         <xsl:text>"</xsl:text>
      </xsl:element>
      <xsl:if test="position()!=last()">&#160;</xsl:if>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match empty elements
   -->

   <xsl:template match="*[not(node())]">
      <xsl:element name="div">
         <xsl:attribute name="class">emptyElement</xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&lt;</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">elementName</xsl:attribute>
            <xsl:value-of select="name(.)"/>
         </xsl:element>
         <xsl:if test="@*">&#160;</xsl:if>
         <xsl:apply-templates select="@*"/>
         <xsl:apply-templates select="." mode="namespace"/>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>/&gt;</xsl:text>
         </xsl:element>
      </xsl:element>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match elements with only text(), they are not closeable
   -->

   <xsl:template match="*[text()][not(* or comment() or processing-instruction())]" priority="10">
      <xsl:element name="div">
         <xsl:attribute name="class">elementWithText</xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">m</xsl:attribute>
            <xsl:text>&lt;</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">elementName</xsl:attribute>
            <xsl:value-of select="name(.)"/>
         </xsl:element>
         <xsl:if test="@*">
            <xsl:text>&#160;</xsl:text>
            <xsl:apply-templates select="@*"/>
         </xsl:if>
         <xsl:apply-templates select="." mode="namespace"/>
         <xsl:element name="span">
            <xsl:attribute name="class">m</xsl:attribute>
            <xsl:text>&gt;</xsl:text>
         </xsl:element>
         <!-- Output the text in this element -->
         <xsl:element name="span">
            <xsl:attribute name="class">text</xsl:attribute>
            <xsl:value-of select="."/>
         </xsl:element>
         <!-- Output closing tag -->
         <xsl:element name="span">
            <xsl:attribute name="class">m</xsl:attribute>
            <xsl:text>&lt;/</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">elementName</xsl:attribute>
            <xsl:value-of select="name(.)"/>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">m</xsl:attribute>
            <xsl:text>&gt;</xsl:text>
         </xsl:element>
      </xsl:element>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->
   <!--
      Match all other elements. All collapsable elements have a hidden form that it will
      collapse to using the jquery code.
   -->

   <xsl:template match="*[node()]">

<xsl:variable name="uniqueId" select="generate-id()"/>
      
      <xsl:element name="div">
         <xsl:attribute name="class">elementHidden</xsl:attribute>
         <xsl:attribute name="id">
            <xsl:value-of select="$uniqueId"/>
         </xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&lt;</xsl:text>
         </xsl:element>
         <xsl:element name="a">
            <xsl:attribute name="href">#</xsl:attribute>
            <xsl:attribute name="class">elementName</xsl:attribute>
            <xsl:value-of select="name(.)"/>
         </xsl:element>
         <xsl:if test="@*">&#160;</xsl:if>
         <xsl:apply-templates select="@*"/>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&gt;</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">text</xsl:attribute>
            <xsl:text>......</xsl:text>
         </xsl:element>
         <!-- closing element tag -->
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&lt;/</xsl:text>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">elementName</xsl:attribute>
            <xsl:value-of select="name(.)"/>
         </xsl:element>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&gt;</xsl:text>
         </xsl:element>
      </xsl:element>


      <xsl:element name="div">
         <xsl:attribute name="class">element</xsl:attribute>
         <xsl:attribute name="id">
            <xsl:value-of select="$uniqueId"/>
         </xsl:attribute>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&lt;</xsl:text>
         </xsl:element>
         <xsl:element name="a">
            <xsl:attribute name="href">#</xsl:attribute>
            <xsl:attribute name="class">elementName</xsl:attribute>
            <xsl:value-of select="name(.)"/>
         </xsl:element>
         <xsl:if test="@*">&#160;</xsl:if>
         <xsl:apply-templates select="@*"/>
         <xsl:apply-templates select="." mode="namespace"/>
         <xsl:element name="span">
            <xsl:attribute name="class">tagDelimiter</xsl:attribute>
            <xsl:text>&gt;</xsl:text>
         </xsl:element>
         <xsl:element name="div">
            <xsl:apply-templates/>
            <!-- closing element tag -->
            <xsl:element name="div">
               <xsl:element name="span">
                  <xsl:attribute name="class">tagDelimiter</xsl:attribute>
                  <xsl:text>&lt;/</xsl:text>
               </xsl:element>
               <xsl:element name="span">
                  <xsl:attribute name="class">elementName</xsl:attribute>
                  <xsl:value-of select="name(.)"/>
               </xsl:element>
               <xsl:element name="span">
                  <xsl:attribute name="class">tagDelimiter</xsl:attribute>
                  <xsl:text>&gt;</xsl:text>
               </xsl:element>
            </xsl:element>
         </xsl:element>
      </xsl:element>
   </xsl:template>

   <!-- -*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-*-* -->

   <xsl:template match="*" mode="namespace">
      <xsl:variable name="context" select="."/>
      <xsl:for-each select="namespace::node()">
         <xsl:variable name="nsuri" select="."/>
         <xsl:variable name="nsprefix" select="name()"/>
         <xsl:choose>
            <xsl:when test="$nsprefix = 'xml'">
               <!-- xml namespace -->
            </xsl:when>
            <xsl:when test="$context/../namespace::node()[name() = $nsprefix and . = $nsuri]">
               <!-- namespace already declared on the parent -->
            </xsl:when>
            <xsl:otherwise>
               <xsl:text>&#160;</xsl:text>
               <xsl:element name="span">
                  <xsl:attribute name="class">attributeName</xsl:attribute>
                  <xsl:text>xmlns</xsl:text>
                  <xsl:if test="$nsprefix">
                     <xsl:text>:</xsl:text>
                     <xsl:value-of select="$nsprefix"/>
                  </xsl:if>
               </xsl:element>
               <xsl:element name="span">
                  <xsl:attribute name="class">m</xsl:attribute>
                  <xsl:text>="</xsl:text>
               </xsl:element>
               <xsl:element name="span">
                  <xsl:attribute name="class">attributeValue</xsl:attribute>
                  <xsl:value-of select="."/>
               </xsl:element>
               <xsl:element name="span">
                  <xsl:attribute name="class">m</xsl:attribute>
                  <xsl:text>"</xsl:text>
               </xsl:element>
            </xsl:otherwise>
         </xsl:choose>
      </xsl:for-each>
   </xsl:template>

</xsl:stylesheet>
